from enum import Enum


class CollectionTypeOptions(str, Enum):
    BOOKS = "Books"
    BOXSETS = "BoxSets"
    HOMEVIDEOS = "HomeVideos"
    MIXED = "Mixed"
    MOVIES = "Movies"
    MUSIC = "Music"
    MUSICVIDEOS = "MusicVideos"
    TVSHOWS = "TvShows"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="LibraryOptionInfoDto")


@_attrs_define
class LibraryOptionInfoDto:
    """Library option info dto.

    Attributes:
        name (Union[Unset, None, str]): Gets or sets name.
        default_enabled (Union[Unset, bool]): Gets or sets a value indicating whether default enabled.
    """

    name: Union[Unset, None, str] = UNSET
    default_enabled: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        default_enabled = self.default_enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if default_enabled is not UNSET:
            field_dict["DefaultEnabled"] = default_enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        default_enabled = d.pop("DefaultEnabled", UNSET)

        library_option_info_dto = cls(
            name=name,
            default_enabled=default_enabled,
        )

        return library_option_info_dto



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..models.sync_play_user_access_type import SyncPlayUserAccessType
from ..models.unrated_item import UnratedItem
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.access_schedule import AccessSchedule


T = TypeVar("T", bound="UserPolicy")


@_attrs_define
class UserPolicy:
    """
    Attributes:
        is_administrator (Union[Unset, bool]): Gets or sets a value indicating whether this instance is administrator.
        is_hidden (Union[Unset, bool]): Gets or sets a value indicating whether this instance is hidden.
        is_disabled (Union[Unset, bool]): Gets or sets a value indicating whether this instance is disabled.
        max_parental_rating (Union[Unset, None, int]): Gets or sets the max parental rating.
        blocked_tags (Union[Unset, None, List[str]]):
        enable_user_preference_access (Union[Unset, bool]):
        access_schedules (Union[Unset, None, List['AccessSchedule']]):
        block_unrated_items (Union[Unset, None, List[UnratedItem]]):
        enable_remote_control_of_other_users (Union[Unset, bool]):
        enable_shared_device_control (Union[Unset, bool]):
        enable_remote_access (Union[Unset, bool]):
        enable_live_tv_management (Union[Unset, bool]):
        enable_live_tv_access (Union[Unset, bool]):
        enable_media_playback (Union[Unset, bool]):
        enable_audio_playback_transcoding (Union[Unset, bool]):
        enable_video_playback_transcoding (Union[Unset, bool]):
        enable_playback_remuxing (Union[Unset, bool]):
        force_remote_source_transcoding (Union[Unset, bool]):
        enable_content_deletion (Union[Unset, bool]):
        enable_content_deletion_from_folders (Union[Unset, None, List[str]]):
        enable_content_downloading (Union[Unset, bool]):
        enable_sync_transcoding (Union[Unset, bool]): Gets or sets a value indicating whether [enable synchronize].
        enable_media_conversion (Union[Unset, bool]):
        enabled_devices (Union[Unset, None, List[str]]):
        enable_all_devices (Union[Unset, bool]):
        enabled_channels (Union[Unset, None, List[str]]):
        enable_all_channels (Union[Unset, bool]):
        enabled_folders (Union[Unset, None, List[str]]):
        enable_all_folders (Union[Unset, bool]):
        invalid_login_attempt_count (Union[Unset, int]):
        login_attempts_before_lockout (Union[Unset, int]):
        max_active_sessions (Union[Unset, int]):
        enable_public_sharing (Union[Unset, bool]):
        blocked_media_folders (Union[Unset, None, List[str]]):
        blocked_channels (Union[Unset, None, List[str]]):
        remote_client_bitrate_limit (Union[Unset, int]):
        authentication_provider_id (Union[Unset, None, str]):
        password_reset_provider_id (Union[Unset, None, str]):
        sync_play_access (Union[Unset, SyncPlayUserAccessType]): Enum SyncPlayUserAccessType.
    """

    is_administrator: Union[Unset, bool] = UNSET
    is_hidden: Union[Unset, bool] = UNSET
    is_disabled: Union[Unset, bool] = UNSET
    max_parental_rating: Union[Unset, None, int] = UNSET
    blocked_tags: Union[Unset, None, List[str]] = UNSET
    enable_user_preference_access: Union[Unset, bool] = UNSET
    access_schedules: Union[Unset, None, List["AccessSchedule"]] = UNSET
    block_unrated_items: Union[Unset, None, List[UnratedItem]] = UNSET
    enable_remote_control_of_other_users: Union[Unset, bool] = UNSET
    enable_shared_device_control: Union[Unset, bool] = UNSET
    enable_remote_access: Union[Unset, bool] = UNSET
    enable_live_tv_management: Union[Unset, bool] = UNSET
    enable_live_tv_access: Union[Unset, bool] = UNSET
    enable_media_playback: Union[Unset, bool] = UNSET
    enable_audio_playback_transcoding: Union[Unset, bool] = UNSET
    enable_video_playback_transcoding: Union[Unset, bool] = UNSET
    enable_playback_remuxing: Union[Unset, bool] = UNSET
    force_remote_source_transcoding: Union[Unset, bool] = UNSET
    enable_content_deletion: Union[Unset, bool] = UNSET
    enable_content_deletion_from_folders: Union[Unset, None, List[str]] = UNSET
    enable_content_downloading: Union[Unset, bool] = UNSET
    enable_sync_transcoding: Union[Unset, bool] = UNSET
    enable_media_conversion: Union[Unset, bool] = UNSET
    enabled_devices: Union[Unset, None, List[str]] = UNSET
    enable_all_devices: Union[Unset, bool] = UNSET
    enabled_channels: Union[Unset, None, List[str]] = UNSET
    enable_all_channels: Union[Unset, bool] = UNSET
    enabled_folders: Union[Unset, None, List[str]] = UNSET
    enable_all_folders: Union[Unset, bool] = UNSET
    invalid_login_attempt_count: Union[Unset, int] = UNSET
    login_attempts_before_lockout: Union[Unset, int] = UNSET
    max_active_sessions: Union[Unset, int] = UNSET
    enable_public_sharing: Union[Unset, bool] = UNSET
    blocked_media_folders: Union[Unset, None, List[str]] = UNSET
    blocked_channels: Union[Unset, None, List[str]] = UNSET
    remote_client_bitrate_limit: Union[Unset, int] = UNSET
    authentication_provider_id: Union[Unset, None, str] = UNSET
    password_reset_provider_id: Union[Unset, None, str] = UNSET
    sync_play_access: Union[Unset, SyncPlayUserAccessType] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        is_administrator = self.is_administrator
        is_hidden = self.is_hidden
        is_disabled = self.is_disabled
        max_parental_rating = self.max_parental_rating
        blocked_tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.blocked_tags, Unset):
            if self.blocked_tags is None:
                blocked_tags = None
            else:
                blocked_tags = self.blocked_tags

        enable_user_preference_access = self.enable_user_preference_access
        access_schedules: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.access_schedules, Unset):
            if self.access_schedules is None:
                access_schedules = None
            else:
                access_schedules = []
                for access_schedules_item_data in self.access_schedules:
                    access_schedules_item = access_schedules_item_data.to_dict()

                    access_schedules.append(access_schedules_item)

        block_unrated_items: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.block_unrated_items, Unset):
            if self.block_unrated_items is None:
                block_unrated_items = None
            else:
                block_unrated_items = []
                for block_unrated_items_item_data in self.block_unrated_items:
                    block_unrated_items_item = block_unrated_items_item_data.value

                    block_unrated_items.append(block_unrated_items_item)

        enable_remote_control_of_other_users = self.enable_remote_control_of_other_users
        enable_shared_device_control = self.enable_shared_device_control
        enable_remote_access = self.enable_remote_access
        enable_live_tv_management = self.enable_live_tv_management
        enable_live_tv_access = self.enable_live_tv_access
        enable_media_playback = self.enable_media_playback
        enable_audio_playback_transcoding = self.enable_audio_playback_transcoding
        enable_video_playback_transcoding = self.enable_video_playback_transcoding
        enable_playback_remuxing = self.enable_playback_remuxing
        force_remote_source_transcoding = self.force_remote_source_transcoding
        enable_content_deletion = self.enable_content_deletion
        enable_content_deletion_from_folders: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.enable_content_deletion_from_folders, Unset):
            if self.enable_content_deletion_from_folders is None:
                enable_content_deletion_from_folders = None
            else:
                enable_content_deletion_from_folders = self.enable_content_deletion_from_folders

        enable_content_downloading = self.enable_content_downloading
        enable_sync_transcoding = self.enable_sync_transcoding
        enable_media_conversion = self.enable_media_conversion
        enabled_devices: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.enabled_devices, Unset):
            if self.enabled_devices is None:
                enabled_devices = None
            else:
                enabled_devices = self.enabled_devices

        enable_all_devices = self.enable_all_devices
        enabled_channels: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.enabled_channels, Unset):
            if self.enabled_channels is None:
                enabled_channels = None
            else:
                enabled_channels = self.enabled_channels

        enable_all_channels = self.enable_all_channels
        enabled_folders: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.enabled_folders, Unset):
            if self.enabled_folders is None:
                enabled_folders = None
            else:
                enabled_folders = self.enabled_folders

        enable_all_folders = self.enable_all_folders
        invalid_login_attempt_count = self.invalid_login_attempt_count
        login_attempts_before_lockout = self.login_attempts_before_lockout
        max_active_sessions = self.max_active_sessions
        enable_public_sharing = self.enable_public_sharing
        blocked_media_folders: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.blocked_media_folders, Unset):
            if self.blocked_media_folders is None:
                blocked_media_folders = None
            else:
                blocked_media_folders = self.blocked_media_folders

        blocked_channels: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.blocked_channels, Unset):
            if self.blocked_channels is None:
                blocked_channels = None
            else:
                blocked_channels = self.blocked_channels

        remote_client_bitrate_limit = self.remote_client_bitrate_limit
        authentication_provider_id = self.authentication_provider_id
        password_reset_provider_id = self.password_reset_provider_id
        sync_play_access: Union[Unset, str] = UNSET
        if not isinstance(self.sync_play_access, Unset):
            sync_play_access = self.sync_play_access.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if is_administrator is not UNSET:
            field_dict["IsAdministrator"] = is_administrator
        if is_hidden is not UNSET:
            field_dict["IsHidden"] = is_hidden
        if is_disabled is not UNSET:
            field_dict["IsDisabled"] = is_disabled
        if max_parental_rating is not UNSET:
            field_dict["MaxParentalRating"] = max_parental_rating
        if blocked_tags is not UNSET:
            field_dict["BlockedTags"] = blocked_tags
        if enable_user_preference_access is not UNSET:
            field_dict["EnableUserPreferenceAccess"] = enable_user_preference_access
        if access_schedules is not UNSET:
            field_dict["AccessSchedules"] = access_schedules
        if block_unrated_items is not UNSET:
            field_dict["BlockUnratedItems"] = block_unrated_items
        if enable_remote_control_of_other_users is not UNSET:
            field_dict["EnableRemoteControlOfOtherUsers"] = enable_remote_control_of_other_users
        if enable_shared_device_control is not UNSET:
            field_dict["EnableSharedDeviceControl"] = enable_shared_device_control
        if enable_remote_access is not UNSET:
            field_dict["EnableRemoteAccess"] = enable_remote_access
        if enable_live_tv_management is not UNSET:
            field_dict["EnableLiveTvManagement"] = enable_live_tv_management
        if enable_live_tv_access is not UNSET:
            field_dict["EnableLiveTvAccess"] = enable_live_tv_access
        if enable_media_playback is not UNSET:
            field_dict["EnableMediaPlayback"] = enable_media_playback
        if enable_audio_playback_transcoding is not UNSET:
            field_dict["EnableAudioPlaybackTranscoding"] = enable_audio_playback_transcoding
        if enable_video_playback_transcoding is not UNSET:
            field_dict["EnableVideoPlaybackTranscoding"] = enable_video_playback_transcoding
        if enable_playback_remuxing is not UNSET:
            field_dict["EnablePlaybackRemuxing"] = enable_playback_remuxing
        if force_remote_source_transcoding is not UNSET:
            field_dict["ForceRemoteSourceTranscoding"] = force_remote_source_transcoding
        if enable_content_deletion is not UNSET:
            field_dict["EnableContentDeletion"] = enable_content_deletion
        if enable_content_deletion_from_folders is not UNSET:
            field_dict["EnableContentDeletionFromFolders"] = enable_content_deletion_from_folders
        if enable_content_downloading is not UNSET:
            field_dict["EnableContentDownloading"] = enable_content_downloading
        if enable_sync_transcoding is not UNSET:
            field_dict["EnableSyncTranscoding"] = enable_sync_transcoding
        if enable_media_conversion is not UNSET:
            field_dict["EnableMediaConversion"] = enable_media_conversion
        if enabled_devices is not UNSET:
            field_dict["EnabledDevices"] = enabled_devices
        if enable_all_devices is not UNSET:
            field_dict["EnableAllDevices"] = enable_all_devices
        if enabled_channels is not UNSET:
            field_dict["EnabledChannels"] = enabled_channels
        if enable_all_channels is not UNSET:
            field_dict["EnableAllChannels"] = enable_all_channels
        if enabled_folders is not UNSET:
            field_dict["EnabledFolders"] = enabled_folders
        if enable_all_folders is not UNSET:
            field_dict["EnableAllFolders"] = enable_all_folders
        if invalid_login_attempt_count is not UNSET:
            field_dict["InvalidLoginAttemptCount"] = invalid_login_attempt_count
        if login_attempts_before_lockout is not UNSET:
            field_dict["LoginAttemptsBeforeLockout"] = login_attempts_before_lockout
        if max_active_sessions is not UNSET:
            field_dict["MaxActiveSessions"] = max_active_sessions
        if enable_public_sharing is not UNSET:
            field_dict["EnablePublicSharing"] = enable_public_sharing
        if blocked_media_folders is not UNSET:
            field_dict["BlockedMediaFolders"] = blocked_media_folders
        if blocked_channels is not UNSET:
            field_dict["BlockedChannels"] = blocked_channels
        if remote_client_bitrate_limit is not UNSET:
            field_dict["RemoteClientBitrateLimit"] = remote_client_bitrate_limit
        if authentication_provider_id is not UNSET:
            field_dict["AuthenticationProviderId"] = authentication_provider_id
        if password_reset_provider_id is not UNSET:
            field_dict["PasswordResetProviderId"] = password_reset_provider_id
        if sync_play_access is not UNSET:
            field_dict["SyncPlayAccess"] = sync_play_access

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.access_schedule import AccessSchedule

        d = src_dict.copy()
        is_administrator = d.pop("IsAdministrator", UNSET)

        is_hidden = d.pop("IsHidden", UNSET)

        is_disabled = d.pop("IsDisabled", UNSET)

        max_parental_rating = d.pop("MaxParentalRating", UNSET)

        blocked_tags = cast(List[str], d.pop("BlockedTags", UNSET))

        enable_user_preference_access = d.pop("EnableUserPreferenceAccess", UNSET)

        access_schedules = []
        _access_schedules = d.pop("AccessSchedules", UNSET)
        for access_schedules_item_data in _access_schedules or []:
            access_schedules_item = AccessSchedule.from_dict(access_schedules_item_data)

            access_schedules.append(access_schedules_item)

        block_unrated_items = []
        _block_unrated_items = d.pop("BlockUnratedItems", UNSET)
        for block_unrated_items_item_data in _block_unrated_items or []:
            block_unrated_items_item = UnratedItem(block_unrated_items_item_data)

            block_unrated_items.append(block_unrated_items_item)

        enable_remote_control_of_other_users = d.pop("EnableRemoteControlOfOtherUsers", UNSET)

        enable_shared_device_control = d.pop("EnableSharedDeviceControl", UNSET)

        enable_remote_access = d.pop("EnableRemoteAccess", UNSET)

        enable_live_tv_management = d.pop("EnableLiveTvManagement", UNSET)

        enable_live_tv_access = d.pop("EnableLiveTvAccess", UNSET)

        enable_media_playback = d.pop("EnableMediaPlayback", UNSET)

        enable_audio_playback_transcoding = d.pop("EnableAudioPlaybackTranscoding", UNSET)

        enable_video_playback_transcoding = d.pop("EnableVideoPlaybackTranscoding", UNSET)

        enable_playback_remuxing = d.pop("EnablePlaybackRemuxing", UNSET)

        force_remote_source_transcoding = d.pop("ForceRemoteSourceTranscoding", UNSET)

        enable_content_deletion = d.pop("EnableContentDeletion", UNSET)

        enable_content_deletion_from_folders = cast(List[str], d.pop("EnableContentDeletionFromFolders", UNSET))

        enable_content_downloading = d.pop("EnableContentDownloading", UNSET)

        enable_sync_transcoding = d.pop("EnableSyncTranscoding", UNSET)

        enable_media_conversion = d.pop("EnableMediaConversion", UNSET)

        enabled_devices = cast(List[str], d.pop("EnabledDevices", UNSET))

        enable_all_devices = d.pop("EnableAllDevices", UNSET)

        enabled_channels = cast(List[str], d.pop("EnabledChannels", UNSET))

        enable_all_channels = d.pop("EnableAllChannels", UNSET)

        enabled_folders = cast(List[str], d.pop("EnabledFolders", UNSET))

        enable_all_folders = d.pop("EnableAllFolders", UNSET)

        invalid_login_attempt_count = d.pop("InvalidLoginAttemptCount", UNSET)

        login_attempts_before_lockout = d.pop("LoginAttemptsBeforeLockout", UNSET)

        max_active_sessions = d.pop("MaxActiveSessions", UNSET)

        enable_public_sharing = d.pop("EnablePublicSharing", UNSET)

        blocked_media_folders = cast(List[str], d.pop("BlockedMediaFolders", UNSET))

        blocked_channels = cast(List[str], d.pop("BlockedChannels", UNSET))

        remote_client_bitrate_limit = d.pop("RemoteClientBitrateLimit", UNSET)

        authentication_provider_id = d.pop("AuthenticationProviderId", UNSET)

        password_reset_provider_id = d.pop("PasswordResetProviderId", UNSET)

        _sync_play_access = d.pop("SyncPlayAccess", UNSET)
        sync_play_access: Union[Unset, SyncPlayUserAccessType]
        if isinstance(_sync_play_access, Unset):
            sync_play_access = UNSET
        else:
            sync_play_access = SyncPlayUserAccessType(_sync_play_access)

        user_policy = cls(
            is_administrator=is_administrator,
            is_hidden=is_hidden,
            is_disabled=is_disabled,
            max_parental_rating=max_parental_rating,
            blocked_tags=blocked_tags,
            enable_user_preference_access=enable_user_preference_access,
            access_schedules=access_schedules,
            block_unrated_items=block_unrated_items,
            enable_remote_control_of_other_users=enable_remote_control_of_other_users,
            enable_shared_device_control=enable_shared_device_control,
            enable_remote_access=enable_remote_access,
            enable_live_tv_management=enable_live_tv_management,
            enable_live_tv_access=enable_live_tv_access,
            enable_media_playback=enable_media_playback,
            enable_audio_playback_transcoding=enable_audio_playback_transcoding,
            enable_video_playback_transcoding=enable_video_playback_transcoding,
            enable_playback_remuxing=enable_playback_remuxing,
            force_remote_source_transcoding=force_remote_source_transcoding,
            enable_content_deletion=enable_content_deletion,
            enable_content_deletion_from_folders=enable_content_deletion_from_folders,
            enable_content_downloading=enable_content_downloading,
            enable_sync_transcoding=enable_sync_transcoding,
            enable_media_conversion=enable_media_conversion,
            enabled_devices=enabled_devices,
            enable_all_devices=enable_all_devices,
            enabled_channels=enabled_channels,
            enable_all_channels=enable_all_channels,
            enabled_folders=enabled_folders,
            enable_all_folders=enable_all_folders,
            invalid_login_attempt_count=invalid_login_attempt_count,
            login_attempts_before_lockout=login_attempts_before_lockout,
            max_active_sessions=max_active_sessions,
            enable_public_sharing=enable_public_sharing,
            blocked_media_folders=blocked_media_folders,
            blocked_channels=blocked_channels,
            remote_client_bitrate_limit=remote_client_bitrate_limit,
            authentication_provider_id=authentication_provider_id,
            password_reset_provider_id=password_reset_provider_id,
            sync_play_access=sync_play_access,
        )

        return user_policy



import datetime
from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="QuickConnectResult")


@_attrs_define
class QuickConnectResult:
    """Stores the state of an quick connect request.

    Attributes:
        authenticated (Union[Unset, bool]): Gets or sets a value indicating whether this request is authorized.
        secret (Union[Unset, str]): Gets the secret value used to uniquely identify this request. Can be used to
            retrieve authentication information.
        code (Union[Unset, str]): Gets the user facing code used so the user can quickly differentiate this request from
            others.
        device_id (Union[Unset, str]): Gets the requesting device id.
        device_name (Union[Unset, str]): Gets the requesting device name.
        app_name (Union[Unset, str]): Gets the requesting app name.
        app_version (Union[Unset, str]): Gets the requesting app version.
        date_added (Union[Unset, datetime.datetime]): Gets or sets the DateTime that this request was created.
    """

    authenticated: Union[Unset, bool] = UNSET
    secret: Union[Unset, str] = UNSET
    code: Union[Unset, str] = UNSET
    device_id: Union[Unset, str] = UNSET
    device_name: Union[Unset, str] = UNSET
    app_name: Union[Unset, str] = UNSET
    app_version: Union[Unset, str] = UNSET
    date_added: Union[Unset, datetime.datetime] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        authenticated = self.authenticated
        secret = self.secret
        code = self.code
        device_id = self.device_id
        device_name = self.device_name
        app_name = self.app_name
        app_version = self.app_version
        date_added: Union[Unset, str] = UNSET
        if not isinstance(self.date_added, Unset):
            date_added = self.date_added.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if authenticated is not UNSET:
            field_dict["Authenticated"] = authenticated
        if secret is not UNSET:
            field_dict["Secret"] = secret
        if code is not UNSET:
            field_dict["Code"] = code
        if device_id is not UNSET:
            field_dict["DeviceId"] = device_id
        if device_name is not UNSET:
            field_dict["DeviceName"] = device_name
        if app_name is not UNSET:
            field_dict["AppName"] = app_name
        if app_version is not UNSET:
            field_dict["AppVersion"] = app_version
        if date_added is not UNSET:
            field_dict["DateAdded"] = date_added

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        authenticated = d.pop("Authenticated", UNSET)

        secret = d.pop("Secret", UNSET)

        code = d.pop("Code", UNSET)

        device_id = d.pop("DeviceId", UNSET)

        device_name = d.pop("DeviceName", UNSET)

        app_name = d.pop("AppName", UNSET)

        app_version = d.pop("AppVersion", UNSET)

        _date_added = d.pop("DateAdded", UNSET)
        date_added: Union[Unset, datetime.datetime]
        if isinstance(_date_added, Unset):
            date_added = UNSET
        else:
            date_added = isoparse(_date_added)

        quick_connect_result = cls(
            authenticated=authenticated,
            secret=secret,
            code=code,
            device_id=device_id,
            device_name=device_name,
            app_name=app_name,
            app_version=app_version,
            date_added=date_added,
        )

        return quick_connect_result



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="MediaUrl")


@_attrs_define
class MediaUrl:
    """
    Attributes:
        url (Union[Unset, None, str]):
        name (Union[Unset, None, str]):
    """

    url: Union[Unset, None, str] = UNSET
    name: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        url = self.url
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if url is not UNSET:
            field_dict["Url"] = url
        if name is not UNSET:
            field_dict["Name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("Url", UNSET)

        name = d.pop("Name", UNSET)

        media_url = cls(
            url=url,
            name=name,
        )

        return media_url



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.library_options import LibraryOptions


T = TypeVar("T", bound="AddVirtualFolderDto")


@_attrs_define
class AddVirtualFolderDto:
    """Add virtual folder dto.

    Attributes:
        library_options (Union[Unset, None, LibraryOptions]):
    """

    library_options: Union[Unset, None, "LibraryOptions"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        library_options: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.library_options, Unset):
            library_options = self.library_options.to_dict() if self.library_options else None

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if library_options is not UNSET:
            field_dict["LibraryOptions"] = library_options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.library_options import LibraryOptions

        d = src_dict.copy()
        _library_options = d.pop("LibraryOptions", UNSET)
        library_options: Union[Unset, None, LibraryOptions]
        if _library_options is None:
            library_options = None
        elif isinstance(_library_options, Unset):
            library_options = UNSET
        else:
            library_options = LibraryOptions.from_dict(_library_options)

        add_virtual_folder_dto = cls(
            library_options=library_options,
        )

        return add_virtual_folder_dto



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="TunerChannelMapping")


@_attrs_define
class TunerChannelMapping:
    """
    Attributes:
        name (Union[Unset, None, str]):
        provider_channel_name (Union[Unset, None, str]):
        provider_channel_id (Union[Unset, None, str]):
        id (Union[Unset, None, str]):
    """

    name: Union[Unset, None, str] = UNSET
    provider_channel_name: Union[Unset, None, str] = UNSET
    provider_channel_id: Union[Unset, None, str] = UNSET
    id: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        provider_channel_name = self.provider_channel_name
        provider_channel_id = self.provider_channel_id
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if provider_channel_name is not UNSET:
            field_dict["ProviderChannelName"] = provider_channel_name
        if provider_channel_id is not UNSET:
            field_dict["ProviderChannelId"] = provider_channel_id
        if id is not UNSET:
            field_dict["Id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        provider_channel_name = d.pop("ProviderChannelName", UNSET)

        provider_channel_id = d.pop("ProviderChannelId", UNSET)

        id = d.pop("Id", UNSET)

        tuner_channel_mapping = cls(
            name=name,
            provider_channel_name=provider_channel_name,
            provider_channel_id=provider_channel_id,
            id=id,
        )

        return tuner_channel_mapping



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    route_item_id: str,
    route_media_source_id: str,
    route_index: int,
    route_format: str,
    *,
    item_id: Union[Unset, None, str] = UNSET,
    media_source_id: Union[Unset, None, str] = UNSET,
    index: Union[Unset, None, int] = UNSET,
    format_: Union[Unset, None, str] = UNSET,
    end_position_ticks: Union[Unset, None, int] = UNSET,
    copy_timestamps: Union[Unset, None, bool] = False,
    add_vtt_time_map: Union[Unset, None, bool] = False,
    start_position_ticks: Union[Unset, None, int] = 0,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    params["itemId"] = item_id

    params["mediaSourceId"] = media_source_id

    params["index"] = index

    params["format"] = format_

    params["endPositionTicks"] = end_position_ticks

    params["copyTimestamps"] = copy_timestamps

    params["addVttTimeMap"] = add_vtt_time_map

    params["startPositionTicks"] = start_position_ticks

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/Videos/{routeItemId}/{routeMediaSourceId}/Subtitles/{routeIndex}/Stream.{routeFormat}".format(
            routeItemId=route_item_id,
            routeMediaSourceId=route_media_source_id,
            routeIndex=route_index,
            routeFormat=route_format,
        ),
        "params": params,
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    route_item_id: str,
    route_media_source_id: str,
    route_index: int,
    route_format: str,
    *,
    client: Union[AuthenticatedClient, Client],
    item_id: Union[Unset, None, str] = UNSET,
    media_source_id: Union[Unset, None, str] = UNSET,
    index: Union[Unset, None, int] = UNSET,
    format_: Union[Unset, None, str] = UNSET,
    end_position_ticks: Union[Unset, None, int] = UNSET,
    copy_timestamps: Union[Unset, None, bool] = False,
    add_vtt_time_map: Union[Unset, None, bool] = False,
    start_position_ticks: Union[Unset, None, int] = 0,
) -> Response[Any]:
    """Gets subtitles in a specified format.

    Args:
        route_item_id (str):
        route_media_source_id (str):
        route_index (int):
        route_format (str):
        item_id (Union[Unset, None, str]):
        media_source_id (Union[Unset, None, str]):
        index (Union[Unset, None, int]):
        format_ (Union[Unset, None, str]):
        end_position_ticks (Union[Unset, None, int]):
        copy_timestamps (Union[Unset, None, bool]):
        add_vtt_time_map (Union[Unset, None, bool]):
        start_position_ticks (Union[Unset, None, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        route_item_id=route_item_id,
        route_media_source_id=route_media_source_id,
        route_index=route_index,
        route_format=route_format,
        item_id=item_id,
        media_source_id=media_source_id,
        index=index,
        format_=format_,
        end_position_ticks=end_position_ticks,
        copy_timestamps=copy_timestamps,
        add_vtt_time_map=add_vtt_time_map,
        start_position_ticks=start_position_ticks,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    route_item_id: str,
    route_media_source_id: str,
    route_index: int,
    route_format: str,
    *,
    client: Union[AuthenticatedClient, Client],
    item_id: Union[Unset, None, str] = UNSET,
    media_source_id: Union[Unset, None, str] = UNSET,
    index: Union[Unset, None, int] = UNSET,
    format_: Union[Unset, None, str] = UNSET,
    end_position_ticks: Union[Unset, None, int] = UNSET,
    copy_timestamps: Union[Unset, None, bool] = False,
    add_vtt_time_map: Union[Unset, None, bool] = False,
    start_position_ticks: Union[Unset, None, int] = 0,
) -> Response[Any]:
    """Gets subtitles in a specified format.

    Args:
        route_item_id (str):
        route_media_source_id (str):
        route_index (int):
        route_format (str):
        item_id (Union[Unset, None, str]):
        media_source_id (Union[Unset, None, str]):
        index (Union[Unset, None, int]):
        format_ (Union[Unset, None, str]):
        end_position_ticks (Union[Unset, None, int]):
        copy_timestamps (Union[Unset, None, bool]):
        add_vtt_time_map (Union[Unset, None, bool]):
        start_position_ticks (Union[Unset, None, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        route_item_id=route_item_id,
        route_media_source_id=route_media_source_id,
        route_index=route_index,
        route_format=route_format,
        item_id=item_id,
        media_source_id=media_source_id,
        index=index,
        format_=format_,
        end_position_ticks=end_position_ticks,
        copy_timestamps=copy_timestamps,
        add_vtt_time_map=add_vtt_time_map,
        start_position_ticks=start_position_ticks,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..models.collection_type_options import CollectionTypeOptions
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.library_options import LibraryOptions


T = TypeVar("T", bound="VirtualFolderInfo")


@_attrs_define
class VirtualFolderInfo:
    """Used to hold information about a user's list of configured virtual folders.

    Attributes:
        name (Union[Unset, None, str]): Gets or sets the name.
        locations (Union[Unset, None, List[str]]): Gets or sets the locations.
        collection_type (Union[Unset, None, CollectionTypeOptions]):
        library_options (Union[Unset, None, LibraryOptions]):
        item_id (Union[Unset, None, str]): Gets or sets the item identifier.
        primary_image_item_id (Union[Unset, None, str]): Gets or sets the primary image item identifier.
        refresh_progress (Union[Unset, None, float]):
        refresh_status (Union[Unset, None, str]):
    """

    name: Union[Unset, None, str] = UNSET
    locations: Union[Unset, None, List[str]] = UNSET
    collection_type: Union[Unset, None, CollectionTypeOptions] = UNSET
    library_options: Union[Unset, None, "LibraryOptions"] = UNSET
    item_id: Union[Unset, None, str] = UNSET
    primary_image_item_id: Union[Unset, None, str] = UNSET
    refresh_progress: Union[Unset, None, float] = UNSET
    refresh_status: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        locations: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.locations, Unset):
            if self.locations is None:
                locations = None
            else:
                locations = self.locations

        collection_type: Union[Unset, None, str] = UNSET
        if not isinstance(self.collection_type, Unset):
            collection_type = self.collection_type.value if self.collection_type else None

        library_options: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.library_options, Unset):
            library_options = self.library_options.to_dict() if self.library_options else None

        item_id = self.item_id
        primary_image_item_id = self.primary_image_item_id
        refresh_progress = self.refresh_progress
        refresh_status = self.refresh_status

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if locations is not UNSET:
            field_dict["Locations"] = locations
        if collection_type is not UNSET:
            field_dict["CollectionType"] = collection_type
        if library_options is not UNSET:
            field_dict["LibraryOptions"] = library_options
        if item_id is not UNSET:
            field_dict["ItemId"] = item_id
        if primary_image_item_id is not UNSET:
            field_dict["PrimaryImageItemId"] = primary_image_item_id
        if refresh_progress is not UNSET:
            field_dict["RefreshProgress"] = refresh_progress
        if refresh_status is not UNSET:
            field_dict["RefreshStatus"] = refresh_status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.library_options import LibraryOptions

        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        locations = cast(List[str], d.pop("Locations", UNSET))

        _collection_type = d.pop("CollectionType", UNSET)
        collection_type: Union[Unset, None, CollectionTypeOptions]
        if _collection_type is None:
            collection_type = None
        elif isinstance(_collection_type, Unset):
            collection_type = UNSET
        else:
            collection_type = CollectionTypeOptions(_collection_type)

        _library_options = d.pop("LibraryOptions", UNSET)
        library_options: Union[Unset, None, LibraryOptions]
        if _library_options is None:
            library_options = None
        elif isinstance(_library_options, Unset):
            library_options = UNSET
        else:
            library_options = LibraryOptions.from_dict(_library_options)

        item_id = d.pop("ItemId", UNSET)

        primary_image_item_id = d.pop("PrimaryImageItemId", UNSET)

        refresh_progress = d.pop("RefreshProgress", UNSET)

        refresh_status = d.pop("RefreshStatus", UNSET)

        virtual_folder_info = cls(
            name=name,
            locations=locations,
            collection_type=collection_type,
            library_options=library_options,
            item_id=item_id,
            primary_image_item_id=primary_image_item_id,
            refresh_progress=refresh_progress,
            refresh_status=refresh_status,
        )

        return virtual_folder_info



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="XbmcMetadataOptions")


@_attrs_define
class XbmcMetadataOptions:
    """
    Attributes:
        user_id (Union[Unset, None, str]):
        release_date_format (Union[Unset, str]):
        save_image_paths_in_nfo (Union[Unset, bool]):
        enable_path_substitution (Union[Unset, bool]):
        enable_extra_thumbs_duplication (Union[Unset, bool]):
    """

    user_id: Union[Unset, None, str] = UNSET
    release_date_format: Union[Unset, str] = UNSET
    save_image_paths_in_nfo: Union[Unset, bool] = UNSET
    enable_path_substitution: Union[Unset, bool] = UNSET
    enable_extra_thumbs_duplication: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id
        release_date_format = self.release_date_format
        save_image_paths_in_nfo = self.save_image_paths_in_nfo
        enable_path_substitution = self.enable_path_substitution
        enable_extra_thumbs_duplication = self.enable_extra_thumbs_duplication

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if user_id is not UNSET:
            field_dict["UserId"] = user_id
        if release_date_format is not UNSET:
            field_dict["ReleaseDateFormat"] = release_date_format
        if save_image_paths_in_nfo is not UNSET:
            field_dict["SaveImagePathsInNfo"] = save_image_paths_in_nfo
        if enable_path_substitution is not UNSET:
            field_dict["EnablePathSubstitution"] = enable_path_substitution
        if enable_extra_thumbs_duplication is not UNSET:
            field_dict["EnableExtraThumbsDuplication"] = enable_extra_thumbs_duplication

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("UserId", UNSET)

        release_date_format = d.pop("ReleaseDateFormat", UNSET)

        save_image_paths_in_nfo = d.pop("SaveImagePathsInNfo", UNSET)

        enable_path_substitution = d.pop("EnablePathSubstitution", UNSET)

        enable_extra_thumbs_duplication = d.pop("EnableExtraThumbsDuplication", UNSET)

        xbmc_metadata_options = cls(
            user_id=user_id,
            release_date_format=release_date_format,
            save_image_paths_in_nfo=save_image_paths_in_nfo,
            enable_path_substitution=enable_path_substitution,
            enable_extra_thumbs_duplication=enable_extra_thumbs_duplication,
        )

        return xbmc_metadata_options



from enum import Enum


class MediaProtocol(str, Enum):
    FILE = "File"
    FTP = "Ftp"
    HTTP = "Http"
    RTMP = "Rtmp"
    RTP = "Rtp"
    RTSP = "Rtsp"
    UDP = "Udp"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.http_header_info import HttpHeaderInfo


T = TypeVar("T", bound="DeviceIdentification")


@_attrs_define
class DeviceIdentification:
    """
    Attributes:
        friendly_name (Union[Unset, str]): Gets or sets the name of the friendly.
        model_number (Union[Unset, str]): Gets or sets the model number.
        serial_number (Union[Unset, str]): Gets or sets the serial number.
        model_name (Union[Unset, str]): Gets or sets the name of the model.
        model_description (Union[Unset, str]): Gets or sets the model description.
        model_url (Union[Unset, str]): Gets or sets the model URL.
        manufacturer (Union[Unset, str]): Gets or sets the manufacturer.
        manufacturer_url (Union[Unset, str]): Gets or sets the manufacturer URL.
        headers (Union[Unset, List['HttpHeaderInfo']]): Gets or sets the headers.
    """

    friendly_name: Union[Unset, str] = UNSET
    model_number: Union[Unset, str] = UNSET
    serial_number: Union[Unset, str] = UNSET
    model_name: Union[Unset, str] = UNSET
    model_description: Union[Unset, str] = UNSET
    model_url: Union[Unset, str] = UNSET
    manufacturer: Union[Unset, str] = UNSET
    manufacturer_url: Union[Unset, str] = UNSET
    headers: Union[Unset, List["HttpHeaderInfo"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        friendly_name = self.friendly_name
        model_number = self.model_number
        serial_number = self.serial_number
        model_name = self.model_name
        model_description = self.model_description
        model_url = self.model_url
        manufacturer = self.manufacturer
        manufacturer_url = self.manufacturer_url
        headers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.headers, Unset):
            headers = []
            for headers_item_data in self.headers:
                headers_item = headers_item_data.to_dict()

                headers.append(headers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if friendly_name is not UNSET:
            field_dict["FriendlyName"] = friendly_name
        if model_number is not UNSET:
            field_dict["ModelNumber"] = model_number
        if serial_number is not UNSET:
            field_dict["SerialNumber"] = serial_number
        if model_name is not UNSET:
            field_dict["ModelName"] = model_name
        if model_description is not UNSET:
            field_dict["ModelDescription"] = model_description
        if model_url is not UNSET:
            field_dict["ModelUrl"] = model_url
        if manufacturer is not UNSET:
            field_dict["Manufacturer"] = manufacturer
        if manufacturer_url is not UNSET:
            field_dict["ManufacturerUrl"] = manufacturer_url
        if headers is not UNSET:
            field_dict["Headers"] = headers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.http_header_info import HttpHeaderInfo

        d = src_dict.copy()
        friendly_name = d.pop("FriendlyName", UNSET)

        model_number = d.pop("ModelNumber", UNSET)

        serial_number = d.pop("SerialNumber", UNSET)

        model_name = d.pop("ModelName", UNSET)

        model_description = d.pop("ModelDescription", UNSET)

        model_url = d.pop("ModelUrl", UNSET)

        manufacturer = d.pop("Manufacturer", UNSET)

        manufacturer_url = d.pop("ManufacturerUrl", UNSET)

        headers = []
        _headers = d.pop("Headers", UNSET)
        for headers_item_data in _headers or []:
            headers_item = HttpHeaderInfo.from_dict(headers_item_data)

            headers.append(headers_item)

        device_identification = cls(
            friendly_name=friendly_name,
            model_number=model_number,
            serial_number=serial_number,
            model_name=model_name,
            model_description=model_description,
            model_url=model_url,
            manufacturer=manufacturer,
            manufacturer_url=manufacturer_url,
            headers=headers,
        )

        return device_identification



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.play_method import PlayMethod
from ...types import UNSET, Response, Unset


def _get_kwargs(
    user_id: str,
    item_id: str,
    *,
    media_source_id: Union[Unset, None, str] = UNSET,
    audio_stream_index: Union[Unset, None, int] = UNSET,
    subtitle_stream_index: Union[Unset, None, int] = UNSET,
    play_method: Union[Unset, None, PlayMethod] = UNSET,
    live_stream_id: Union[Unset, None, str] = UNSET,
    play_session_id: Union[Unset, None, str] = UNSET,
    can_seek: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    params["mediaSourceId"] = media_source_id

    params["audioStreamIndex"] = audio_stream_index

    params["subtitleStreamIndex"] = subtitle_stream_index

    json_play_method: Union[Unset, None, str] = UNSET
    if not isinstance(play_method, Unset):
        json_play_method = play_method.value if play_method else None

    params["playMethod"] = json_play_method

    params["liveStreamId"] = live_stream_id

    params["playSessionId"] = play_session_id

    params["canSeek"] = can_seek

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "post",
        "url": "/Users/{userId}/PlayingItems/{itemId}".format(
            userId=user_id,
            itemId=item_id,
        ),
        "params": params,
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.NO_CONTENT:
        return None
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        return None
    if response.status_code == HTTPStatus.FORBIDDEN:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    user_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    media_source_id: Union[Unset, None, str] = UNSET,
    audio_stream_index: Union[Unset, None, int] = UNSET,
    subtitle_stream_index: Union[Unset, None, int] = UNSET,
    play_method: Union[Unset, None, PlayMethod] = UNSET,
    live_stream_id: Union[Unset, None, str] = UNSET,
    play_session_id: Union[Unset, None, str] = UNSET,
    can_seek: Union[Unset, None, bool] = False,
) -> Response[Any]:
    """Reports that a user has begun playing an item.

    Args:
        user_id (str):
        item_id (str):
        media_source_id (Union[Unset, None, str]):
        audio_stream_index (Union[Unset, None, int]):
        subtitle_stream_index (Union[Unset, None, int]):
        play_method (Union[Unset, None, PlayMethod]):
        live_stream_id (Union[Unset, None, str]):
        play_session_id (Union[Unset, None, str]):
        can_seek (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        item_id=item_id,
        media_source_id=media_source_id,
        audio_stream_index=audio_stream_index,
        subtitle_stream_index=subtitle_stream_index,
        play_method=play_method,
        live_stream_id=live_stream_id,
        play_session_id=play_session_id,
        can_seek=can_seek,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    user_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    media_source_id: Union[Unset, None, str] = UNSET,
    audio_stream_index: Union[Unset, None, int] = UNSET,
    subtitle_stream_index: Union[Unset, None, int] = UNSET,
    play_method: Union[Unset, None, PlayMethod] = UNSET,
    live_stream_id: Union[Unset, None, str] = UNSET,
    play_session_id: Union[Unset, None, str] = UNSET,
    can_seek: Union[Unset, None, bool] = False,
) -> Response[Any]:
    """Reports that a user has begun playing an item.

    Args:
        user_id (str):
        item_id (str):
        media_source_id (Union[Unset, None, str]):
        audio_stream_index (Union[Unset, None, int]):
        subtitle_stream_index (Union[Unset, None, int]):
        play_method (Union[Unset, None, PlayMethod]):
        live_stream_id (Union[Unset, None, str]):
        play_session_id (Union[Unset, None, str]):
        can_seek (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        item_id=item_id,
        media_source_id=media_source_id,
        audio_stream_index=audio_stream_index,
        subtitle_stream_index=subtitle_stream_index,
        play_method=play_method,
        live_stream_id=live_stream_id,
        play_session_id=play_session_id,
        can_seek=can_seek,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.file_system_entry_type import FileSystemEntryType
from ..types import UNSET, Unset

T = TypeVar("T", bound="FileSystemEntryInfo")


@_attrs_define
class FileSystemEntryInfo:
    """Class FileSystemEntryInfo.

    Attributes:
        name (Union[Unset, str]): Gets the name.
        path (Union[Unset, str]): Gets the path.
        type (Union[Unset, FileSystemEntryType]): Enum FileSystemEntryType.
    """

    name: Union[Unset, str] = UNSET
    path: Union[Unset, str] = UNSET
    type: Union[Unset, FileSystemEntryType] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        path = self.path
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if path is not UNSET:
            field_dict["Path"] = path
        if type is not UNSET:
            field_dict["Type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        path = d.pop("Path", UNSET)

        _type = d.pop("Type", UNSET)
        type: Union[Unset, FileSystemEntryType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = FileSystemEntryType(_type)

        file_system_entry_info = cls(
            name=name,
            path=path,
            type=type,
        )

        return file_system_entry_info



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="CreatePlaylistDto")


@_attrs_define
class CreatePlaylistDto:
    """Create new playlist dto.

    Attributes:
        name (Union[Unset, None, str]): Gets or sets the name of the new playlist.
        ids (Union[Unset, List[str]]): Gets or sets item ids to add to the playlist.
        user_id (Union[Unset, None, str]): Gets or sets the user id.
        media_type (Union[Unset, None, str]): Gets or sets the media type.
    """

    name: Union[Unset, None, str] = UNSET
    ids: Union[Unset, List[str]] = UNSET
    user_id: Union[Unset, None, str] = UNSET
    media_type: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.ids, Unset):
            ids = self.ids

        user_id = self.user_id
        media_type = self.media_type

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if ids is not UNSET:
            field_dict["Ids"] = ids
        if user_id is not UNSET:
            field_dict["UserId"] = user_id
        if media_type is not UNSET:
            field_dict["MediaType"] = media_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        ids = cast(List[str], d.pop("Ids", UNSET))

        user_id = d.pop("UserId", UNSET)

        media_type = d.pop("MediaType", UNSET)

        create_playlist_dto = cls(
            name=name,
            ids=ids,
            user_id=user_id,
            media_type=media_type,
        )

        return create_playlist_dto



from enum import Enum


class ChannelMediaContentType(str, Enum):
    CLIP = "Clip"
    EPISODE = "Episode"
    MOVIE = "Movie"
    MOVIEEXTRA = "MovieExtra"
    PODCAST = "Podcast"
    SONG = "Song"
    TRAILER = "Trailer"
    TVEXTRA = "TvExtra"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class TaskCompletionStatus(str, Enum):
    ABORTED = "Aborted"
    CANCELLED = "Cancelled"
    COMPLETED = "Completed"
    FAILED = "Failed"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="JoinGroupRequestDto")


@_attrs_define
class JoinGroupRequestDto:
    """Class JoinGroupRequestDto.

    Attributes:
        group_id (Union[Unset, str]): Gets or sets the group identifier.
    """

    group_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        group_id = self.group_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if group_id is not UNSET:
            field_dict["GroupId"] = group_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        group_id = d.pop("GroupId", UNSET)

        join_group_request_dto = cls(
            group_id=group_id,
        )

        return join_group_request_dto



from enum import Enum


class EmbeddedSubtitleOptions(str, Enum):
    ALLOWALL = "AllowAll"
    ALLOWIMAGE = "AllowImage"
    ALLOWNONE = "AllowNone"
    ALLOWTEXT = "AllowText"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.plugin_status import PluginStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="PluginInfo")


@_attrs_define
class PluginInfo:
    """This is a serializable stub class that is used by the api to provide information about installed plugins.

    Attributes:
        name (Union[Unset, str]): Gets or sets the name.
        version (Union[Unset, str]): Gets or sets the version.
        configuration_file_name (Union[Unset, None, str]): Gets or sets the name of the configuration file.
        description (Union[Unset, str]): Gets or sets the description.
        id (Union[Unset, str]): Gets or sets the unique id.
        can_uninstall (Union[Unset, bool]): Gets or sets a value indicating whether the plugin can be uninstalled.
        has_image (Union[Unset, bool]): Gets or sets a value indicating whether this plugin has a valid image.
        status (Union[Unset, PluginStatus]): Plugin load status.
    """

    name: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    configuration_file_name: Union[Unset, None, str] = UNSET
    description: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    can_uninstall: Union[Unset, bool] = UNSET
    has_image: Union[Unset, bool] = UNSET
    status: Union[Unset, PluginStatus] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        version = self.version
        configuration_file_name = self.configuration_file_name
        description = self.description
        id = self.id
        can_uninstall = self.can_uninstall
        has_image = self.has_image
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if version is not UNSET:
            field_dict["Version"] = version
        if configuration_file_name is not UNSET:
            field_dict["ConfigurationFileName"] = configuration_file_name
        if description is not UNSET:
            field_dict["Description"] = description
        if id is not UNSET:
            field_dict["Id"] = id
        if can_uninstall is not UNSET:
            field_dict["CanUninstall"] = can_uninstall
        if has_image is not UNSET:
            field_dict["HasImage"] = has_image
        if status is not UNSET:
            field_dict["Status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        version = d.pop("Version", UNSET)

        configuration_file_name = d.pop("ConfigurationFileName", UNSET)

        description = d.pop("Description", UNSET)

        id = d.pop("Id", UNSET)

        can_uninstall = d.pop("CanUninstall", UNSET)

        has_image = d.pop("HasImage", UNSET)

        _status = d.pop("Status", UNSET)
        status: Union[Unset, PluginStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = PluginStatus(_status)

        plugin_info = cls(
            name=name,
            version=version,
            configuration_file_name=configuration_file_name,
            description=description,
            id=id,
            can_uninstall=can_uninstall,
            has_image=has_image,
            status=status,
        )

        return plugin_info



import datetime
from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.trailer_info_provider_ids import TrailerInfoProviderIds


T = TypeVar("T", bound="TrailerInfo")


@_attrs_define
class TrailerInfo:
    """
    Attributes:
        name (Union[Unset, None, str]): Gets or sets the name.
        original_title (Union[Unset, None, str]): Gets or sets the original title.
        path (Union[Unset, None, str]): Gets or sets the path.
        metadata_language (Union[Unset, None, str]): Gets or sets the metadata language.
        metadata_country_code (Union[Unset, None, str]): Gets or sets the metadata country code.
        provider_ids (Union[Unset, None, TrailerInfoProviderIds]): Gets or sets the provider ids.
        year (Union[Unset, None, int]): Gets or sets the year.
        index_number (Union[Unset, None, int]):
        parent_index_number (Union[Unset, None, int]):
        premiere_date (Union[Unset, None, datetime.datetime]):
        is_automated (Union[Unset, bool]):
    """

    name: Union[Unset, None, str] = UNSET
    original_title: Union[Unset, None, str] = UNSET
    path: Union[Unset, None, str] = UNSET
    metadata_language: Union[Unset, None, str] = UNSET
    metadata_country_code: Union[Unset, None, str] = UNSET
    provider_ids: Union[Unset, None, "TrailerInfoProviderIds"] = UNSET
    year: Union[Unset, None, int] = UNSET
    index_number: Union[Unset, None, int] = UNSET
    parent_index_number: Union[Unset, None, int] = UNSET
    premiere_date: Union[Unset, None, datetime.datetime] = UNSET
    is_automated: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        original_title = self.original_title
        path = self.path
        metadata_language = self.metadata_language
        metadata_country_code = self.metadata_country_code
        provider_ids: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.provider_ids, Unset):
            provider_ids = self.provider_ids.to_dict() if self.provider_ids else None

        year = self.year
        index_number = self.index_number
        parent_index_number = self.parent_index_number
        premiere_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.premiere_date, Unset):
            premiere_date = self.premiere_date.isoformat() if self.premiere_date else None

        is_automated = self.is_automated

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if original_title is not UNSET:
            field_dict["OriginalTitle"] = original_title
        if path is not UNSET:
            field_dict["Path"] = path
        if metadata_language is not UNSET:
            field_dict["MetadataLanguage"] = metadata_language
        if metadata_country_code is not UNSET:
            field_dict["MetadataCountryCode"] = metadata_country_code
        if provider_ids is not UNSET:
            field_dict["ProviderIds"] = provider_ids
        if year is not UNSET:
            field_dict["Year"] = year
        if index_number is not UNSET:
            field_dict["IndexNumber"] = index_number
        if parent_index_number is not UNSET:
            field_dict["ParentIndexNumber"] = parent_index_number
        if premiere_date is not UNSET:
            field_dict["PremiereDate"] = premiere_date
        if is_automated is not UNSET:
            field_dict["IsAutomated"] = is_automated

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.trailer_info_provider_ids import TrailerInfoProviderIds

        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        original_title = d.pop("OriginalTitle", UNSET)

        path = d.pop("Path", UNSET)

        metadata_language = d.pop("MetadataLanguage", UNSET)

        metadata_country_code = d.pop("MetadataCountryCode", UNSET)

        _provider_ids = d.pop("ProviderIds", UNSET)
        provider_ids: Union[Unset, None, TrailerInfoProviderIds]
        if _provider_ids is None:
            provider_ids = None
        elif isinstance(_provider_ids, Unset):
            provider_ids = UNSET
        else:
            provider_ids = TrailerInfoProviderIds.from_dict(_provider_ids)

        year = d.pop("Year", UNSET)

        index_number = d.pop("IndexNumber", UNSET)

        parent_index_number = d.pop("ParentIndexNumber", UNSET)

        _premiere_date = d.pop("PremiereDate", UNSET)
        premiere_date: Union[Unset, None, datetime.datetime]
        if _premiere_date is None:
            premiere_date = None
        elif isinstance(_premiere_date, Unset):
            premiere_date = UNSET
        else:
            premiere_date = isoparse(_premiere_date)

        is_automated = d.pop("IsAutomated", UNSET)

        trailer_info = cls(
            name=name,
            original_title=original_title,
            path=path,
            metadata_language=metadata_language,
            metadata_country_code=metadata_country_code,
            provider_ids=provider_ids,
            year=year,
            index_number=index_number,
            parent_index_number=parent_index_number,
            premiere_date=premiere_date,
            is_automated=is_automated,
        )

        return trailer_info



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="SessionUserInfo")


@_attrs_define
class SessionUserInfo:
    """Class SessionUserInfo.

    Attributes:
        user_id (Union[Unset, str]): Gets or sets the user identifier.
        user_name (Union[Unset, None, str]): Gets or sets the name of the user.
    """

    user_id: Union[Unset, str] = UNSET
    user_name: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id
        user_name = self.user_name

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if user_id is not UNSET:
            field_dict["UserId"] = user_id
        if user_name is not UNSET:
            field_dict["UserName"] = user_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("UserId", UNSET)

        user_name = d.pop("UserName", UNSET)

        session_user_info = cls(
            user_id=user_id,
            user_name=user_name,
        )

        return session_user_info



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..models.live_tv_service_status import LiveTvServiceStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="LiveTvServiceInfo")


@_attrs_define
class LiveTvServiceInfo:
    """Class ServiceInfo.

    Attributes:
        name (Union[Unset, None, str]): Gets or sets the name.
        home_page_url (Union[Unset, None, str]): Gets or sets the home page URL.
        status (Union[Unset, LiveTvServiceStatus]):
        status_message (Union[Unset, None, str]): Gets or sets the status message.
        version (Union[Unset, None, str]): Gets or sets the version.
        has_update_available (Union[Unset, bool]): Gets or sets a value indicating whether this instance has update
            available.
        is_visible (Union[Unset, bool]): Gets or sets a value indicating whether this instance is visible.
        tuners (Union[Unset, None, List[str]]):
    """

    name: Union[Unset, None, str] = UNSET
    home_page_url: Union[Unset, None, str] = UNSET
    status: Union[Unset, LiveTvServiceStatus] = UNSET
    status_message: Union[Unset, None, str] = UNSET
    version: Union[Unset, None, str] = UNSET
    has_update_available: Union[Unset, bool] = UNSET
    is_visible: Union[Unset, bool] = UNSET
    tuners: Union[Unset, None, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        home_page_url = self.home_page_url
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        status_message = self.status_message
        version = self.version
        has_update_available = self.has_update_available
        is_visible = self.is_visible
        tuners: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tuners, Unset):
            if self.tuners is None:
                tuners = None
            else:
                tuners = self.tuners

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if home_page_url is not UNSET:
            field_dict["HomePageUrl"] = home_page_url
        if status is not UNSET:
            field_dict["Status"] = status
        if status_message is not UNSET:
            field_dict["StatusMessage"] = status_message
        if version is not UNSET:
            field_dict["Version"] = version
        if has_update_available is not UNSET:
            field_dict["HasUpdateAvailable"] = has_update_available
        if is_visible is not UNSET:
            field_dict["IsVisible"] = is_visible
        if tuners is not UNSET:
            field_dict["Tuners"] = tuners

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        home_page_url = d.pop("HomePageUrl", UNSET)

        _status = d.pop("Status", UNSET)
        status: Union[Unset, LiveTvServiceStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = LiveTvServiceStatus(_status)

        status_message = d.pop("StatusMessage", UNSET)

        version = d.pop("Version", UNSET)

        has_update_available = d.pop("HasUpdateAvailable", UNSET)

        is_visible = d.pop("IsVisible", UNSET)

        tuners = cast(List[str], d.pop("Tuners", UNSET))

        live_tv_service_info = cls(
            name=name,
            home_page_url=home_page_url,
            status=status,
            status_message=status_message,
            version=version,
            has_update_available=has_update_available,
            is_visible=is_visible,
            tuners=tuners,
        )

        return live_tv_service_info



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.codec_profile import CodecProfile
    from ..models.container_profile import ContainerProfile
    from ..models.device_identification import DeviceIdentification
    from ..models.direct_play_profile import DirectPlayProfile
    from ..models.response_profile import ResponseProfile
    from ..models.subtitle_profile import SubtitleProfile
    from ..models.transcoding_profile import TranscodingProfile
    from ..models.xml_attribute import XmlAttribute


T = TypeVar("T", bound="DeviceProfile")


@_attrs_define
class DeviceProfile:
    """A MediaBrowser.Model.Dlna.DeviceProfile represents a set of metadata which determines which content a certain device
    is able to play.
    <br />
    Specifically, it defines the supported <see
    cref="P:MediaBrowser.Model.Dlna.DeviceProfile.ContainerProfiles">containers</see> and
    <see cref="P:MediaBrowser.Model.Dlna.DeviceProfile.CodecProfiles">codecs</see> (video and/or audio, including codec
    profiles and levels)
    the device is able to direct play (without transcoding or remuxing),
    as well as which <see cref="P:MediaBrowser.Model.Dlna.DeviceProfile.TranscodingProfiles">containers/codecs to
    transcode to</see> in case it isn't.

        Attributes:
            name (Union[Unset, None, str]): Gets or sets the name of this device profile.
            id (Union[Unset, None, str]): Gets or sets the Id.
            identification (Union[Unset, None, DeviceIdentification]):
            friendly_name (Union[Unset, None, str]): Gets or sets the friendly name of the device profile, which can be
                shown to users.
            manufacturer (Union[Unset, None, str]): Gets or sets the manufacturer of the device which this profile
                represents.
            manufacturer_url (Union[Unset, None, str]): Gets or sets an url for the manufacturer of the device which this
                profile represents.
            model_name (Union[Unset, None, str]): Gets or sets the model name of the device which this profile represents.
            model_description (Union[Unset, None, str]): Gets or sets the model description of the device which this profile
                represents.
            model_number (Union[Unset, None, str]): Gets or sets the model number of the device which this profile
                represents.
            model_url (Union[Unset, None, str]): Gets or sets the ModelUrl.
            serial_number (Union[Unset, None, str]): Gets or sets the serial number of the device which this profile
                represents.
            enable_album_art_in_didl (Union[Unset, bool]): Gets or sets a value indicating whether EnableAlbumArtInDidl.
            enable_single_album_art_limit (Union[Unset, bool]): Gets or sets a value indicating whether
                EnableSingleAlbumArtLimit.
            enable_single_subtitle_limit (Union[Unset, bool]): Gets or sets a value indicating whether
                EnableSingleSubtitleLimit.
            supported_media_types (Union[Unset, str]): Gets or sets the SupportedMediaTypes.
            user_id (Union[Unset, None, str]): Gets or sets the UserId.
            album_art_pn (Union[Unset, None, str]): Gets or sets the AlbumArtPn.
            max_album_art_width (Union[Unset, None, int]): Gets or sets the MaxAlbumArtWidth.
            max_album_art_height (Union[Unset, None, int]): Gets or sets the MaxAlbumArtHeight.
            max_icon_width (Union[Unset, None, int]): Gets or sets the maximum allowed width of embedded icons.
            max_icon_height (Union[Unset, None, int]): Gets or sets the maximum allowed height of embedded icons.
            max_streaming_bitrate (Union[Unset, None, int]): Gets or sets the maximum allowed bitrate for all streamed
                content.
            max_static_bitrate (Union[Unset, None, int]): Gets or sets the maximum allowed bitrate for statically streamed
                content (= direct played files).
            music_streaming_transcoding_bitrate (Union[Unset, None, int]): Gets or sets the maximum allowed bitrate for
                transcoded music streams.
            max_static_music_bitrate (Union[Unset, None, int]): Gets or sets the maximum allowed bitrate for statically
                streamed (= direct played) music files.
            sony_aggregation_flags (Union[Unset, None, str]): Gets or sets the content of the aggregationFlags element in
                the urn:schemas-sonycom:av namespace.
            protocol_info (Union[Unset, None, str]): Gets or sets the ProtocolInfo.
            timeline_offset_seconds (Union[Unset, int]): Gets or sets the TimelineOffsetSeconds.
            requires_plain_video_items (Union[Unset, bool]): Gets or sets a value indicating whether
                RequiresPlainVideoItems.
            requires_plain_folders (Union[Unset, bool]): Gets or sets a value indicating whether RequiresPlainFolders.
            enable_ms_media_receiver_registrar (Union[Unset, bool]): Gets or sets a value indicating whether
                EnableMSMediaReceiverRegistrar.
            ignore_transcode_byte_range_requests (Union[Unset, bool]): Gets or sets a value indicating whether
                IgnoreTranscodeByteRangeRequests.
            xml_root_attributes (Union[Unset, List['XmlAttribute']]): Gets or sets the XmlRootAttributes.
            direct_play_profiles (Union[Unset, List['DirectPlayProfile']]): Gets or sets the direct play profiles.
            transcoding_profiles (Union[Unset, List['TranscodingProfile']]): Gets or sets the transcoding profiles.
            container_profiles (Union[Unset, List['ContainerProfile']]): Gets or sets the container profiles.
            codec_profiles (Union[Unset, List['CodecProfile']]): Gets or sets the codec profiles.
            response_profiles (Union[Unset, List['ResponseProfile']]): Gets or sets the ResponseProfiles.
            subtitle_profiles (Union[Unset, List['SubtitleProfile']]): Gets or sets the subtitle profiles.
    """

    name: Union[Unset, None, str] = UNSET
    id: Union[Unset, None, str] = UNSET
    identification: Union[Unset, None, "DeviceIdentification"] = UNSET
    friendly_name: Union[Unset, None, str] = UNSET
    manufacturer: Union[Unset, None, str] = UNSET
    manufacturer_url: Union[Unset, None, str] = UNSET
    model_name: Union[Unset, None, str] = UNSET
    model_description: Union[Unset, None, str] = UNSET
    model_number: Union[Unset, None, str] = UNSET
    model_url: Union[Unset, None, str] = UNSET
    serial_number: Union[Unset, None, str] = UNSET
    enable_album_art_in_didl: Union[Unset, bool] = False
    enable_single_album_art_limit: Union[Unset, bool] = False
    enable_single_subtitle_limit: Union[Unset, bool] = False
    supported_media_types: Union[Unset, str] = UNSET
    user_id: Union[Unset, None, str] = UNSET
    album_art_pn: Union[Unset, None, str] = UNSET
    max_album_art_width: Union[Unset, None, int] = UNSET
    max_album_art_height: Union[Unset, None, int] = UNSET
    max_icon_width: Union[Unset, None, int] = UNSET
    max_icon_height: Union[Unset, None, int] = UNSET
    max_streaming_bitrate: Union[Unset, None, int] = UNSET
    max_static_bitrate: Union[Unset, None, int] = UNSET
    music_streaming_transcoding_bitrate: Union[Unset, None, int] = UNSET
    max_static_music_bitrate: Union[Unset, None, int] = UNSET
    sony_aggregation_flags: Union[Unset, None, str] = UNSET
    protocol_info: Union[Unset, None, str] = UNSET
    timeline_offset_seconds: Union[Unset, int] = 0
    requires_plain_video_items: Union[Unset, bool] = False
    requires_plain_folders: Union[Unset, bool] = False
    enable_ms_media_receiver_registrar: Union[Unset, bool] = False
    ignore_transcode_byte_range_requests: Union[Unset, bool] = False
    xml_root_attributes: Union[Unset, List["XmlAttribute"]] = UNSET
    direct_play_profiles: Union[Unset, List["DirectPlayProfile"]] = UNSET
    transcoding_profiles: Union[Unset, List["TranscodingProfile"]] = UNSET
    container_profiles: Union[Unset, List["ContainerProfile"]] = UNSET
    codec_profiles: Union[Unset, List["CodecProfile"]] = UNSET
    response_profiles: Union[Unset, List["ResponseProfile"]] = UNSET
    subtitle_profiles: Union[Unset, List["SubtitleProfile"]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        id = self.id
        identification: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.identification, Unset):
            identification = self.identification.to_dict() if self.identification else None

        friendly_name = self.friendly_name
        manufacturer = self.manufacturer
        manufacturer_url = self.manufacturer_url
        model_name = self.model_name
        model_description = self.model_description
        model_number = self.model_number
        model_url = self.model_url
        serial_number = self.serial_number
        enable_album_art_in_didl = self.enable_album_art_in_didl
        enable_single_album_art_limit = self.enable_single_album_art_limit
        enable_single_subtitle_limit = self.enable_single_subtitle_limit
        supported_media_types = self.supported_media_types
        user_id = self.user_id
        album_art_pn = self.album_art_pn
        max_album_art_width = self.max_album_art_width
        max_album_art_height = self.max_album_art_height
        max_icon_width = self.max_icon_width
        max_icon_height = self.max_icon_height
        max_streaming_bitrate = self.max_streaming_bitrate
        max_static_bitrate = self.max_static_bitrate
        music_streaming_transcoding_bitrate = self.music_streaming_transcoding_bitrate
        max_static_music_bitrate = self.max_static_music_bitrate
        sony_aggregation_flags = self.sony_aggregation_flags
        protocol_info = self.protocol_info
        timeline_offset_seconds = self.timeline_offset_seconds
        requires_plain_video_items = self.requires_plain_video_items
        requires_plain_folders = self.requires_plain_folders
        enable_ms_media_receiver_registrar = self.enable_ms_media_receiver_registrar
        ignore_transcode_byte_range_requests = self.ignore_transcode_byte_range_requests
        xml_root_attributes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.xml_root_attributes, Unset):
            xml_root_attributes = []
            for xml_root_attributes_item_data in self.xml_root_attributes:
                xml_root_attributes_item = xml_root_attributes_item_data.to_dict()

                xml_root_attributes.append(xml_root_attributes_item)

        direct_play_profiles: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.direct_play_profiles, Unset):
            direct_play_profiles = []
            for direct_play_profiles_item_data in self.direct_play_profiles:
                direct_play_profiles_item = direct_play_profiles_item_data.to_dict()

                direct_play_profiles.append(direct_play_profiles_item)

        transcoding_profiles: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.transcoding_profiles, Unset):
            transcoding_profiles = []
            for transcoding_profiles_item_data in self.transcoding_profiles:
                transcoding_profiles_item = transcoding_profiles_item_data.to_dict()

                transcoding_profiles.append(transcoding_profiles_item)

        container_profiles: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.container_profiles, Unset):
            container_profiles = []
            for container_profiles_item_data in self.container_profiles:
                container_profiles_item = container_profiles_item_data.to_dict()

                container_profiles.append(container_profiles_item)

        codec_profiles: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.codec_profiles, Unset):
            codec_profiles = []
            for codec_profiles_item_data in self.codec_profiles:
                codec_profiles_item = codec_profiles_item_data.to_dict()

                codec_profiles.append(codec_profiles_item)

        response_profiles: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.response_profiles, Unset):
            response_profiles = []
            for response_profiles_item_data in self.response_profiles:
                response_profiles_item = response_profiles_item_data.to_dict()

                response_profiles.append(response_profiles_item)

        subtitle_profiles: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.subtitle_profiles, Unset):
            subtitle_profiles = []
            for subtitle_profiles_item_data in self.subtitle_profiles:
                subtitle_profiles_item = subtitle_profiles_item_data.to_dict()

                subtitle_profiles.append(subtitle_profiles_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["Name"] = name
        if id is not UNSET:
            field_dict["Id"] = id
        if identification is not UNSET:
            field_dict["Identification"] = identification
        if friendly_name is not UNSET:
            field_dict["FriendlyName"] = friendly_name
        if manufacturer is not UNSET:
            field_dict["Manufacturer"] = manufacturer
        if manufacturer_url is not UNSET:
            field_dict["ManufacturerUrl"] = manufacturer_url
        if model_name is not UNSET:
            field_dict["ModelName"] = model_name
        if model_description is not UNSET:
            field_dict["ModelDescription"] = model_description
        if model_number is not UNSET:
            field_dict["ModelNumber"] = model_number
        if model_url is not UNSET:
            field_dict["ModelUrl"] = model_url
        if serial_number is not UNSET:
            field_dict["SerialNumber"] = serial_number
        if enable_album_art_in_didl is not UNSET:
            field_dict["EnableAlbumArtInDidl"] = enable_album_art_in_didl
        if enable_single_album_art_limit is not UNSET:
            field_dict["EnableSingleAlbumArtLimit"] = enable_single_album_art_limit
        if enable_single_subtitle_limit is not UNSET:
            field_dict["EnableSingleSubtitleLimit"] = enable_single_subtitle_limit
        if supported_media_types is not UNSET:
            field_dict["SupportedMediaTypes"] = supported_media_types
        if user_id is not UNSET:
            field_dict["UserId"] = user_id
        if album_art_pn is not UNSET:
            field_dict["AlbumArtPn"] = album_art_pn
        if max_album_art_width is not UNSET:
            field_dict["MaxAlbumArtWidth"] = max_album_art_width
        if max_album_art_height is not UNSET:
            field_dict["MaxAlbumArtHeight"] = max_album_art_height
        if max_icon_width is not UNSET:
            field_dict["MaxIconWidth"] = max_icon_width
        if max_icon_height is not UNSET:
            field_dict["MaxIconHeight"] = max_icon_height
        if max_streaming_bitrate is not UNSET:
            field_dict["MaxStreamingBitrate"] = max_streaming_bitrate
        if max_static_bitrate is not UNSET:
            field_dict["MaxStaticBitrate"] = max_static_bitrate
        if music_streaming_transcoding_bitrate is not UNSET:
            field_dict["MusicStreamingTranscodingBitrate"] = music_streaming_transcoding_bitrate
        if max_static_music_bitrate is not UNSET:
            field_dict["MaxStaticMusicBitrate"] = max_static_music_bitrate
        if sony_aggregation_flags is not UNSET:
            field_dict["SonyAggregationFlags"] = sony_aggregation_flags
        if protocol_info is not UNSET:
            field_dict["ProtocolInfo"] = protocol_info
        if timeline_offset_seconds is not UNSET:
            field_dict["TimelineOffsetSeconds"] = timeline_offset_seconds
        if requires_plain_video_items is not UNSET:
            field_dict["RequiresPlainVideoItems"] = requires_plain_video_items
        if requires_plain_folders is not UNSET:
            field_dict["RequiresPlainFolders"] = requires_plain_folders
        if enable_ms_media_receiver_registrar is not UNSET:
            field_dict["EnableMSMediaReceiverRegistrar"] = enable_ms_media_receiver_registrar
        if ignore_transcode_byte_range_requests is not UNSET:
            field_dict["IgnoreTranscodeByteRangeRequests"] = ignore_transcode_byte_range_requests
        if xml_root_attributes is not UNSET:
            field_dict["XmlRootAttributes"] = xml_root_attributes
        if direct_play_profiles is not UNSET:
            field_dict["DirectPlayProfiles"] = direct_play_profiles
        if transcoding_profiles is not UNSET:
            field_dict["TranscodingProfiles"] = transcoding_profiles
        if container_profiles is not UNSET:
            field_dict["ContainerProfiles"] = container_profiles
        if codec_profiles is not UNSET:
            field_dict["CodecProfiles"] = codec_profiles
        if response_profiles is not UNSET:
            field_dict["ResponseProfiles"] = response_profiles
        if subtitle_profiles is not UNSET:
            field_dict["SubtitleProfiles"] = subtitle_profiles

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.codec_profile import CodecProfile
        from ..models.container_profile import ContainerProfile
        from ..models.device_identification import DeviceIdentification
        from ..models.direct_play_profile import DirectPlayProfile
        from ..models.response_profile import ResponseProfile
        from ..models.subtitle_profile import SubtitleProfile
        from ..models.transcoding_profile import TranscodingProfile
        from ..models.xml_attribute import XmlAttribute

        d = src_dict.copy()
        name = d.pop("Name", UNSET)

        id = d.pop("Id", UNSET)

        _identification = d.pop("Identification", UNSET)
        identification: Union[Unset, None, DeviceIdentification]
        if _identification is None:
            identification = None
        elif isinstance(_identification, Unset):
            identification = UNSET
        else:
            identification = DeviceIdentification.from_dict(_identification)

        friendly_name = d.pop("FriendlyName", UNSET)

        manufacturer = d.pop("Manufacturer", UNSET)

        manufacturer_url = d.pop("ManufacturerUrl", UNSET)

        model_name = d.pop("ModelName", UNSET)

        model_description = d.pop("ModelDescription", UNSET)

        model_number = d.pop("ModelNumber", UNSET)

        model_url = d.pop("ModelUrl", UNSET)

        serial_number = d.pop("SerialNumber", UNSET)

        enable_album_art_in_didl = d.pop("EnableAlbumArtInDidl", UNSET)

        enable_single_album_art_limit = d.pop("EnableSingleAlbumArtLimit", UNSET)

        enable_single_subtitle_limit = d.pop("EnableSingleSubtitleLimit", UNSET)

        supported_media_types = d.pop("SupportedMediaTypes", UNSET)

        user_id = d.pop("UserId", UNSET)

        album_art_pn = d.pop("AlbumArtPn", UNSET)

        max_album_art_width = d.pop("MaxAlbumArtWidth", UNSET)

        max_album_art_height = d.pop("MaxAlbumArtHeight", UNSET)

        max_icon_width = d.pop("MaxIconWidth", UNSET)

        max_icon_height = d.pop("MaxIconHeight", UNSET)

        max_streaming_bitrate = d.pop("MaxStreamingBitrate", UNSET)

        max_static_bitrate = d.pop("MaxStaticBitrate", UNSET)

        music_streaming_transcoding_bitrate = d.pop("MusicStreamingTranscodingBitrate", UNSET)

        max_static_music_bitrate = d.pop("MaxStaticMusicBitrate", UNSET)

        sony_aggregation_flags = d.pop("SonyAggregationFlags", UNSET)

        protocol_info = d.pop("ProtocolInfo", UNSET)

        timeline_offset_seconds = d.pop("TimelineOffsetSeconds", UNSET)

        requires_plain_video_items = d.pop("RequiresPlainVideoItems", UNSET)

        requires_plain_folders = d.pop("RequiresPlainFolders", UNSET)

        enable_ms_media_receiver_registrar = d.pop("EnableMSMediaReceiverRegistrar", UNSET)

        ignore_transcode_byte_range_requests = d.pop("IgnoreTranscodeByteRangeRequests", UNSET)

        xml_root_attributes = []
        _xml_root_attributes = d.pop("XmlRootAttributes", UNSET)
        for xml_root_attributes_item_data in _xml_root_attributes or []:
            xml_root_attributes_item = XmlAttribute.from_dict(xml_root_attributes_item_data)

            xml_root_attributes.append(xml_root_attributes_item)

        direct_play_profiles = []
        _direct_play_profiles = d.pop("DirectPlayProfiles", UNSET)
        for direct_play_profiles_item_data in _direct_play_profiles or []:
            direct_play_profiles_item = DirectPlayProfile.from_dict(direct_play_profiles_item_data)

            direct_play_profiles.append(direct_play_profiles_item)

        transcoding_profiles = []
        _transcoding_profiles = d.pop("TranscodingProfiles", UNSET)
        for transcoding_profiles_item_data in _transcoding_profiles or []:
            transcoding_profiles_item = TranscodingProfile.from_dict(transcoding_profiles_item_data)

            transcoding_profiles.append(transcoding_profiles_item)

        container_profiles = []
        _container_profiles = d.pop("ContainerProfiles", UNSET)
        for container_profiles_item_data in _container_profiles or []:
            container_profiles_item = ContainerProfile.from_dict(container_profiles_item_data)

            container_profiles.append(container_profiles_item)

        codec_profiles = []
        _codec_profiles = d.pop("CodecProfiles", UNSET)
        for codec_profiles_item_data in _codec_profiles or []:
            codec_profiles_item = CodecProfile.from_dict(codec_profiles_item_data)

            codec_profiles.append(codec_profiles_item)

        response_profiles = []
        _response_profiles = d.pop("ResponseProfiles", UNSET)
        for response_profiles_item_data in _response_profiles or []:
            response_profiles_item = ResponseProfile.from_dict(response_profiles_item_data)

            response_profiles.append(response_profiles_item)

        subtitle_profiles = []
        _subtitle_profiles = d.pop("SubtitleProfiles", UNSET)
        for subtitle_profiles_item_data in _subtitle_profiles or []:
            subtitle_profiles_item = SubtitleProfile.from_dict(subtitle_profiles_item_data)

            subtitle_profiles.append(subtitle_profiles_item)

        device_profile = cls(
            name=name,
            id=id,
            identification=identification,
            friendly_name=friendly_name,
            manufacturer=manufacturer,
            manufacturer_url=manufacturer_url,
            model_name=model_name,
            model_description=model_description,
            model_number=model_number,
            model_url=model_url,
            serial_number=serial_number,
            enable_album_art_in_didl=enable_album_art_in_didl,
            enable_single_album_art_limit=enable_single_album_art_limit,
            enable_single_subtitle_limit=enable_single_subtitle_limit,
            supported_media_types=supported_media_types,
            user_id=user_id,
            album_art_pn=album_art_pn,
            max_album_art_width=max_album_art_width,
            max_album_art_height=max_album_art_height,
            max_icon_width=max_icon_width,
            max_icon_height=max_icon_height,
            max_streaming_bitrate=max_streaming_bitrate,
            max_static_bitrate=max_static_bitrate,
            music_streaming_transcoding_bitrate=music_streaming_transcoding_bitrate,
            max_static_music_bitrate=max_static_music_bitrate,
            sony_aggregation_flags=sony_aggregation_flags,
            protocol_info=protocol_info,
            timeline_offset_seconds=timeline_offset_seconds,
            requires_plain_video_items=requires_plain_video_items,
            requires_plain_folders=requires_plain_folders,
            enable_ms_media_receiver_registrar=enable_ms_media_receiver_registrar,
            ignore_transcode_byte_range_requests=ignore_transcode_byte_range_requests,
            xml_root_attributes=xml_root_attributes,
            direct_play_profiles=direct_play_profiles,
            transcoding_profiles=transcoding_profiles,
            container_profiles=container_profiles,
            codec_profiles=codec_profiles,
            response_profiles=response_profiles,
            subtitle_profiles=subtitle_profiles,
        )

        return device_profile



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.image_type import ImageType
from ..models.item_fields import ItemFields
from ..models.sort_order import SortOrder
from ..types import UNSET, Unset

T = TypeVar("T", bound="GetProgramsDto")


@_attrs_define
class GetProgramsDto:
    """Get programs dto.

    Attributes:
        channel_ids (Union[Unset, List[str]]): Gets or sets the channels to return guide information for.
        user_id (Union[Unset, str]): Gets or sets optional. Filter by user id.
        min_start_date (Union[Unset, None, datetime.datetime]): Gets or sets the minimum premiere start date.
            Optional.
        has_aired (Union[Unset, None, bool]): Gets or sets filter by programs that have completed airing, or not.
            Optional.
        is_airing (Union[Unset, None, bool]): Gets or sets filter by programs that are currently airing, or not.
            Optional.
        max_start_date (Union[Unset, None, datetime.datetime]): Gets or sets the maximum premiere start date.
            Optional.
        min_end_date (Union[Unset, None, datetime.datetime]): Gets or sets the minimum premiere end date.
            Optional.
        max_end_date (Union[Unset, None, datetime.datetime]): Gets or sets the maximum premiere end date.
            Optional.
        is_movie (Union[Unset, None, bool]): Gets or sets filter for movies.
            Optional.
        is_series (Union[Unset, None, bool]): Gets or sets filter for series.
            Optional.
        is_news (Union[Unset, None, bool]): Gets or sets filter for news.
            Optional.
        is_kids (Union[Unset, None, bool]): Gets or sets filter for kids.
            Optional.
        is_sports (Union[Unset, None, bool]): Gets or sets filter for sports.
            Optional.
        start_index (Union[Unset, None, int]): Gets or sets the record index to start at. All items with a lower index
            will be dropped from the results.
            Optional.
        limit (Union[Unset, None, int]): Gets or sets the maximum number of records to return.
            Optional.
        sort_by (Union[Unset, List[str]]): Gets or sets specify one or more sort orders, comma delimited. Options: Name,
            StartDate.
            Optional.
        sort_order (Union[Unset, List[SortOrder]]): Gets or sets sort Order - Ascending,Descending.
        genres (Union[Unset, List[str]]): Gets or sets the genres to return guide information for.
        genre_ids (Union[Unset, List[str]]): Gets or sets the genre ids to return guide information for.
        enable_images (Union[Unset, None, bool]): Gets or sets include image information in output.
            Optional.
        enable_total_record_count (Union[Unset, bool]): Gets or sets a value indicating whether retrieve total record
            count.
        image_type_limit (Union[Unset, None, int]): Gets or sets the max number of images to return, per image type.
            Optional.
        enable_image_types (Union[Unset, List[ImageType]]): Gets or sets the image types to include in the output.
            Optional.
        enable_user_data (Union[Unset, None, bool]): Gets or sets include user data.
            Optional.
        series_timer_id (Union[Unset, None, str]): Gets or sets filter by series timer id.
            Optional.
        library_series_id (Union[Unset, str]): Gets or sets filter by library series id.
            Optional.
        fields (Union[Unset, List[ItemFields]]): Gets or sets specify additional fields of information to return in the
            output. This allows multiple, comma delimited. Options: Budget, Chapters, DateCreated, Genres, HomePageUrl,
            IndexOptions, MediaStreams, Overview, ParentId, Path, People, ProviderIds, PrimaryImageAspectRatio, Revenue,
            SortName, Studios, Taglines.
            Optional.
    """

    channel_ids: Union[Unset, List[str]] = UNSET
    user_id: Union[Unset, str] = UNSET
    min_start_date: Union[Unset, None, datetime.datetime] = UNSET
    has_aired: Union[Unset, None, bool] = UNSET
    is_airing: Union[Unset, None, bool] = UNSET
    max_start_date: Union[Unset, None, datetime.datetime] = UNSET
    min_end_date: Union[Unset, None, datetime.datetime] = UNSET
    max_end_date: Union[Unset, None, datetime.datetime] = UNSET
    is_movie: Union[Unset, None, bool] = UNSET
    is_series: Union[Unset, None, bool] = UNSET
    is_news: Union[Unset, None, bool] = UNSET
    is_kids: Union[Unset, None, bool] = UNSET
    is_sports: Union[Unset, None, bool] = UNSET
    start_index: Union[Unset, None, int] = UNSET
    limit: Union[Unset, None, int] = UNSET
    sort_by: Union[Unset, List[str]] = UNSET
    sort_order: Union[Unset, List[SortOrder]] = UNSET
    genres: Union[Unset, List[str]] = UNSET
    genre_ids: Union[Unset, List[str]] = UNSET
    enable_images: Union[Unset, None, bool] = UNSET
    enable_total_record_count: Union[Unset, bool] = UNSET
    image_type_limit: Union[Unset, None, int] = UNSET
    enable_image_types: Union[Unset, List[ImageType]] = UNSET
    enable_user_data: Union[Unset, None, bool] = UNSET
    series_timer_id: Union[Unset, None, str] = UNSET
    library_series_id: Union[Unset, str] = UNSET
    fields: Union[Unset, List[ItemFields]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        channel_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.channel_ids, Unset):
            channel_ids = self.channel_ids

        user_id = self.user_id
        min_start_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.min_start_date, Unset):
            min_start_date = self.min_start_date.isoformat() if self.min_start_date else None

        has_aired = self.has_aired
        is_airing = self.is_airing
        max_start_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.max_start_date, Unset):
            max_start_date = self.max_start_date.isoformat() if self.max_start_date else None

        min_end_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.min_end_date, Unset):
            min_end_date = self.min_end_date.isoformat() if self.min_end_date else None

        max_end_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.max_end_date, Unset):
            max_end_date = self.max_end_date.isoformat() if self.max_end_date else None

        is_movie = self.is_movie
        is_series = self.is_series
        is_news = self.is_news
        is_kids = self.is_kids
        is_sports = self.is_sports
        start_index = self.start_index
        limit = self.limit
        sort_by: Union[Unset, List[str]] = UNSET
        if not isinstance(self.sort_by, Unset):
            sort_by = self.sort_by

        sort_order: Union[Unset, List[str]] = UNSET
        if not isinstance(self.sort_order, Unset):
            sort_order = []
            for sort_order_item_data in self.sort_order:
                sort_order_item = sort_order_item_data.value

                sort_order.append(sort_order_item)

        genres: Union[Unset, List[str]] = UNSET
        if not isinstance(self.genres, Unset):
            genres = self.genres

        genre_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.genre_ids, Unset):
            genre_ids = self.genre_ids

        enable_images = self.enable_images
        enable_total_record_count = self.enable_total_record_count
        image_type_limit = self.image_type_limit
        enable_image_types: Union[Unset, List[str]] = UNSET
        if not isinstance(self.enable_image_types, Unset):
            enable_image_types = []
            for enable_image_types_item_data in self.enable_image_types:
                enable_image_types_item = enable_image_types_item_data.value

                enable_image_types.append(enable_image_types_item)

        enable_user_data = self.enable_user_data
        series_timer_id = self.series_timer_id
        library_series_id = self.library_series_id
        fields: Union[Unset, List[str]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = []
            for fields_item_data in self.fields:
                fields_item = fields_item_data.value

                fields.append(fields_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if channel_ids is not UNSET:
            field_dict["ChannelIds"] = channel_ids
        if user_id is not UNSET:
            field_dict["UserId"] = user_id
        if min_start_date is not UNSET:
            field_dict["MinStartDate"] = min_start_date
        if has_aired is not UNSET:
            field_dict["HasAired"] = has_aired
        if is_airing is not UNSET:
            field_dict["IsAiring"] = is_airing
        if max_start_date is not UNSET:
            field_dict["MaxStartDate"] = max_start_date
        if min_end_date is not UNSET:
            field_dict["MinEndDate"] = min_end_date
        if max_end_date is not UNSET:
            field_dict["MaxEndDate"] = max_end_date
        if is_movie is not UNSET:
            field_dict["IsMovie"] = is_movie
        if is_series is not UNSET:
            field_dict["IsSeries"] = is_series
        if is_news is not UNSET:
            field_dict["IsNews"] = is_news
        if is_kids is not UNSET:
            field_dict["IsKids"] = is_kids
        if is_sports is not UNSET:
            field_dict["IsSports"] = is_sports
        if start_index is not UNSET:
            field_dict["StartIndex"] = start_index
        if limit is not UNSET:
            field_dict["Limit"] = limit
        if sort_by is not UNSET:
            field_dict["SortBy"] = sort_by
        if sort_order is not UNSET:
            field_dict["SortOrder"] = sort_order
        if genres is not UNSET:
            field_dict["Genres"] = genres
        if genre_ids is not UNSET:
            field_dict["GenreIds"] = genre_ids
        if enable_images is not UNSET:
            field_dict["EnableImages"] = enable_images
        if enable_total_record_count is not UNSET:
            field_dict["EnableTotalRecordCount"] = enable_total_record_count
        if image_type_limit is not UNSET:
            field_dict["ImageTypeLimit"] = image_type_limit
        if enable_image_types is not UNSET:
            field_dict["EnableImageTypes"] = enable_image_types
        if enable_user_data is not UNSET:
            field_dict["EnableUserData"] = enable_user_data
        if series_timer_id is not UNSET:
            field_dict["SeriesTimerId"] = series_timer_id
        if library_series_id is not UNSET:
            field_dict["LibrarySeriesId"] = library_series_id
        if fields is not UNSET:
            field_dict["Fields"] = fields

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        channel_ids = cast(List[str], d.pop("ChannelIds", UNSET))

        user_id = d.pop("UserId", UNSET)

        _min_start_date = d.pop("MinStartDate", UNSET)
        min_start_date: Union[Unset, None, datetime.datetime]
        if _min_start_date is None:
            min_start_date = None
        elif isinstance(_min_start_date, Unset):
            min_start_date = UNSET
        else:
            min_start_date = isoparse(_min_start_date)

        has_aired = d.pop("HasAired", UNSET)

        is_airing = d.pop("IsAiring", UNSET)

        _max_start_date = d.pop("MaxStartDate", UNSET)
        max_start_date: Union[Unset, None, datetime.datetime]
        if _max_start_date is None:
            max_start_date = None
        elif isinstance(_max_start_date, Unset):
            max_start_date = UNSET
        else:
            max_start_date = isoparse(_max_start_date)

        _min_end_date = d.pop("MinEndDate", UNSET)
        min_end_date: Union[Unset, None, datetime.datetime]
        if _min_end_date is None:
            min_end_date = None
        elif isinstance(_min_end_date, Unset):
            min_end_date = UNSET
        else:
            min_end_date = isoparse(_min_end_date)

        _max_end_date = d.pop("MaxEndDate", UNSET)
        max_end_date: Union[Unset, None, datetime.datetime]
        if _max_end_date is None:
            max_end_date = None
        elif isinstance(_max_end_date, Unset):
            max_end_date = UNSET
        else:
            max_end_date = isoparse(_max_end_date)

        is_movie = d.pop("IsMovie", UNSET)

        is_series = d.pop("IsSeries", UNSET)

        is_news = d.pop("IsNews", UNSET)

        is_kids = d.pop("IsKids", UNSET)

        is_sports = d.pop("IsSports", UNSET)

        start_index = d.pop("StartIndex", UNSET)

        limit = d.pop("Limit", UNSET)

        sort_by = cast(List[str], d.pop("SortBy", UNSET))

        sort_order = []
        _sort_order = d.pop("SortOrder", UNSET)
        for sort_order_item_data in _sort_order or []:
            sort_order_item = SortOrder(sort_order_item_data)

            sort_order.append(sort_order_item)

        genres = cast(List[str], d.pop("Genres", UNSET))

        genre_ids = cast(List[str], d.pop("GenreIds", UNSET))

        enable_images = d.pop("EnableImages", UNSET)

        enable_total_record_count = d.pop("EnableTotalRecordCount", UNSET)

        image_type_limit = d.pop("ImageTypeLimit", UNSET)

        enable_image_types = []
        _enable_image_types = d.pop("EnableImageTypes", UNSET)
        for enable_image_types_item_data in _enable_image_types or []:
            enable_image_types_item = ImageType(enable_image_types_item_data)

            enable_image_types.append(enable_image_types_item)

        enable_user_data = d.pop("EnableUserData", UNSET)

        series_timer_id = d.pop("SeriesTimerId", UNSET)

        library_series_id = d.pop("LibrarySeriesId", UNSET)

        fields = []
        _fields = d.pop("Fields", UNSET)
        for fields_item_data in _fields or []:
            fields_item = ItemFields(fields_item_data)

            fields.append(fields_item)

        get_programs_dto = cls(
            channel_ids=channel_ids,
            user_id=user_id,
            min_start_date=min_start_date,
            has_aired=has_aired,
            is_airing=is_airing,
            max_start_date=max_start_date,
            min_end_date=min_end_date,
            max_end_date=max_end_date,
            is_movie=is_movie,
            is_series=is_series,
            is_news=is_news,
            is_kids=is_kids,
            is_sports=is_sports,
            start_index=start_index,
            limit=limit,
            sort_by=sort_by,
            sort_order=sort_order,
            genres=genres,
            genre_ids=genre_ids,
            enable_images=enable_images,
            enable_total_record_count=enable_total_record_count,
            image_type_limit=image_type_limit,
            enable_image_types=enable_image_types,
            enable_user_data=enable_user_data,
            series_timer_id=series_timer_id,
            library_series_id=library_series_id,
            fields=fields,
        )

        return get_programs_dto



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.theme_media_result import ThemeMediaResult


T = TypeVar("T", bound="AllThemeMediaResult")


@_attrs_define
class AllThemeMediaResult:
    """
    Attributes:
        theme_videos_result (Union[Unset, None, ThemeMediaResult]): Class ThemeMediaResult.
        theme_songs_result (Union[Unset, None, ThemeMediaResult]): Class ThemeMediaResult.
        soundtrack_songs_result (Union[Unset, None, ThemeMediaResult]): Class ThemeMediaResult.
    """

    theme_videos_result: Union[Unset, None, "ThemeMediaResult"] = UNSET
    theme_songs_result: Union[Unset, None, "ThemeMediaResult"] = UNSET
    soundtrack_songs_result: Union[Unset, None, "ThemeMediaResult"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        theme_videos_result: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.theme_videos_result, Unset):
            theme_videos_result = self.theme_videos_result.to_dict() if self.theme_videos_result else None

        theme_songs_result: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.theme_songs_result, Unset):
            theme_songs_result = self.theme_songs_result.to_dict() if self.theme_songs_result else None

        soundtrack_songs_result: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.soundtrack_songs_result, Unset):
            soundtrack_songs_result = self.soundtrack_songs_result.to_dict() if self.soundtrack_songs_result else None

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if theme_videos_result is not UNSET:
            field_dict["ThemeVideosResult"] = theme_videos_result
        if theme_songs_result is not UNSET:
            field_dict["ThemeSongsResult"] = theme_songs_result
        if soundtrack_songs_result is not UNSET:
            field_dict["SoundtrackSongsResult"] = soundtrack_songs_result

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.theme_media_result import ThemeMediaResult

        d = src_dict.copy()
        _theme_videos_result = d.pop("ThemeVideosResult", UNSET)
        theme_videos_result: Union[Unset, None, ThemeMediaResult]
        if _theme_videos_result is None:
            theme_videos_result = None
        elif isinstance(_theme_videos_result, Unset):
            theme_videos_result = UNSET
        else:
            theme_videos_result = ThemeMediaResult.from_dict(_theme_videos_result)

        _theme_songs_result = d.pop("ThemeSongsResult", UNSET)
        theme_songs_result: Union[Unset, None, ThemeMediaResult]
        if _theme_songs_result is None:
            theme_songs_result = None
        elif isinstance(_theme_songs_result, Unset):
            theme_songs_result = UNSET
        else:
            theme_songs_result = ThemeMediaResult.from_dict(_theme_songs_result)

        _soundtrack_songs_result = d.pop("SoundtrackSongsResult", UNSET)
        soundtrack_songs_result: Union[Unset, None, ThemeMediaResult]
        if _soundtrack_songs_result is None:
            soundtrack_songs_result = None
        elif isinstance(_soundtrack_songs_result, Unset):
            soundtrack_songs_result = UNSET
        else:
            soundtrack_songs_result = ThemeMediaResult.from_dict(_soundtrack_songs_result)

        all_theme_media_result = cls(
            theme_videos_result=theme_videos_result,
            theme_songs_result=theme_songs_result,
            soundtrack_songs_result=soundtrack_songs_result,
        )

        return all_theme_media_result



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..models.play_method import PlayMethod
from ..models.repeat_mode import RepeatMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="PlayerStateInfo")


@_attrs_define
class PlayerStateInfo:
    """
    Attributes:
        position_ticks (Union[Unset, None, int]): Gets or sets the now playing position ticks.
        can_seek (Union[Unset, bool]): Gets or sets a value indicating whether this instance can seek.
        is_paused (Union[Unset, bool]): Gets or sets a value indicating whether this instance is paused.
        is_muted (Union[Unset, bool]): Gets or sets a value indicating whether this instance is muted.
        volume_level (Union[Unset, None, int]): Gets or sets the volume level.
        audio_stream_index (Union[Unset, None, int]): Gets or sets the index of the now playing audio stream.
        subtitle_stream_index (Union[Unset, None, int]): Gets or sets the index of the now playing subtitle stream.
        media_source_id (Union[Unset, None, str]): Gets or sets the now playing media version identifier.
        play_method (Union[Unset, None, PlayMethod]):
        repeat_mode (Union[Unset, RepeatMode]):
        live_stream_id (Union[Unset, None, str]): Gets or sets the now playing live stream identifier.
    """

    position_ticks: Union[Unset, None, int] = UNSET
    can_seek: Union[Unset, bool] = UNSET
    is_paused: Union[Unset, bool] = UNSET
    is_muted: Union[Unset, bool] = UNSET
    volume_level: Union[Unset, None, int] = UNSET
    audio_stream_index: Union[Unset, None, int] = UNSET
    subtitle_stream_index: Union[Unset, None, int] = UNSET
    media_source_id: Union[Unset, None, str] = UNSET
    play_method: Union[Unset, None, PlayMethod] = UNSET
    repeat_mode: Union[Unset, RepeatMode] = UNSET
    live_stream_id: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        position_ticks = self.position_ticks
        can_seek = self.can_seek
        is_paused = self.is_paused
        is_muted = self.is_muted
        volume_level = self.volume_level
        audio_stream_index = self.audio_stream_index
        subtitle_stream_index = self.subtitle_stream_index
        media_source_id = self.media_source_id
        play_method: Union[Unset, None, str] = UNSET
        if not isinstance(self.play_method, Unset):
            play_method = self.play_method.value if self.play_method else None

        repeat_mode: Union[Unset, str] = UNSET
        if not isinstance(self.repeat_mode, Unset):
            repeat_mode = self.repeat_mode.value

        live_stream_id = self.live_stream_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if position_ticks is not UNSET:
            field_dict["PositionTicks"] = position_ticks
        if can_seek is not UNSET:
            field_dict["CanSeek"] = can_seek
        if is_paused is not UNSET:
            field_dict["IsPaused"] = is_paused
        if is_muted is not UNSET:
            field_dict["IsMuted"] = is_muted
        if volume_level is not UNSET:
            field_dict["VolumeLevel"] = volume_level
        if audio_stream_index is not UNSET:
            field_dict["AudioStreamIndex"] = audio_stream_index
        if subtitle_stream_index is not UNSET:
            field_dict["SubtitleStreamIndex"] = subtitle_stream_index
        if media_source_id is not UNSET:
            field_dict["MediaSourceId"] = media_source_id
        if play_method is not UNSET:
            field_dict["PlayMethod"] = play_method
        if repeat_mode is not UNSET:
            field_dict["RepeatMode"] = repeat_mode
        if live_stream_id is not UNSET:
            field_dict["LiveStreamId"] = live_stream_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        position_ticks = d.pop("PositionTicks", UNSET)

        can_seek = d.pop("CanSeek", UNSET)

        is_paused = d.pop("IsPaused", UNSET)

        is_muted = d.pop("IsMuted", UNSET)

        volume_level = d.pop("VolumeLevel", UNSET)

        audio_stream_index = d.pop("AudioStreamIndex", UNSET)

        subtitle_stream_index = d.pop("SubtitleStreamIndex", UNSET)

        media_source_id = d.pop("MediaSourceId", UNSET)

        _play_method = d.pop("PlayMethod", UNSET)
        play_method: Union[Unset, None, PlayMethod]
        if _play_method is None:
            play_method = None
        elif isinstance(_play_method, Unset):
            play_method = UNSET
        else:
            play_method = PlayMethod(_play_method)

        _repeat_mode = d.pop("RepeatMode", UNSET)
        repeat_mode: Union[Unset, RepeatMode]
        if isinstance(_repeat_mode, Unset):
            repeat_mode = UNSET
        else:
            repeat_mode = RepeatMode(_repeat_mode)

        live_stream_id = d.pop("LiveStreamId", UNSET)

        player_state_info = cls(
            position_ticks=position_ticks,
            can_seek=can_seek,
            is_paused=is_paused,
            is_muted=is_muted,
            volume_level=volume_level,
            audio_stream_index=audio_stream_index,
            subtitle_stream_index=subtitle_stream_index,
            media_source_id=media_source_id,
            play_method=play_method,
            repeat_mode=repeat_mode,
            live_stream_id=live_stream_id,
        )

        return player_state_info



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="EncodingOptions")


@_attrs_define
class EncodingOptions:
    """
    Attributes:
        encoding_thread_count (Union[Unset, int]):
        transcoding_temp_path (Union[Unset, None, str]):
        fallback_font_path (Union[Unset, None, str]):
        enable_fallback_font (Union[Unset, bool]):
        down_mix_audio_boost (Union[Unset, float]):
        max_muxing_queue_size (Union[Unset, int]):
        enable_throttling (Union[Unset, bool]):
        throttle_delay_seconds (Union[Unset, int]):
        hardware_acceleration_type (Union[Unset, None, str]):
        encoder_app_path (Union[Unset, None, str]): Gets or sets the FFmpeg path as set by the user via the UI.
        encoder_app_path_display (Union[Unset, None, str]): Gets or sets the current FFmpeg path being used by the
            system and displayed on the transcode page.
        vaapi_device (Union[Unset, None, str]):
        enable_tonemapping (Union[Unset, bool]):
        enable_vpp_tonemapping (Union[Unset, bool]):
        tonemapping_algorithm (Union[Unset, None, str]):
        tonemapping_mode (Union[Unset, None, str]):
        tonemapping_range (Union[Unset, None, str]):
        tonemapping_desat (Union[Unset, float]):
        tonemapping_peak (Union[Unset, float]):
        tonemapping_param (Union[Unset, float]):
        vpp_tonemapping_brightness (Union[Unset, float]):
        vpp_tonemapping_contrast (Union[Unset, float]):
        h264_crf (Union[Unset, int]):
        h265_crf (Union[Unset, int]):
        encoder_preset (Union[Unset, None, str]):
        deinterlace_double_rate (Union[Unset, bool]):
        deinterlace_method (Union[Unset, None, str]):
        enable_decoding_color_depth_10_hevc (Union[Unset, bool]):
        enable_decoding_color_depth_10_vp_9 (Union[Unset, bool]):
        enable_enhanced_nvdec_decoder (Union[Unset, bool]):
        prefer_system_native_hw_decoder (Union[Unset, bool]):
        enable_intel_low_power_h264_hw_encoder (Union[Unset, bool]):
        enable_intel_low_power_hevc_hw_encoder (Union[Unset, bool]):
        enable_hardware_encoding (Union[Unset, bool]):
        allow_hevc_encoding (Union[Unset, bool]):
        enable_subtitle_extraction (Union[Unset, bool]):
        hardware_decoding_codecs (Union[Unset, None, List[str]]):
        allow_on_demand_metadata_based_keyframe_extraction_for_extensions (Union[Unset, None, List[str]]):
    """

    encoding_thread_count: Union[Unset, int] = UNSET
    transcoding_temp_path: Union[Unset, None, str] = UNSET
    fallback_font_path: Union[Unset, None, str] = UNSET
    enable_fallback_font: Union[Unset, bool] = UNSET
    down_mix_audio_boost: Union[Unset, float] = UNSET
    max_muxing_queue_size: Union[Unset, int] = UNSET
    enable_throttling: Union[Unset, bool] = UNSET
    throttle_delay_seconds: Union[Unset, int] = UNSET
    hardware_acceleration_type: Union[Unset, None, str] = UNSET
    encoder_app_path: Union[Unset, None, str] = UNSET
    encoder_app_path_display: Union[Unset, None, str] = UNSET
    vaapi_device: Union[Unset, None, str] = UNSET
    enable_tonemapping: Union[Unset, bool] = UNSET
    enable_vpp_tonemapping: Union[Unset, bool] = UNSET
    tonemapping_algorithm: Union[Unset, None, str] = UNSET
    tonemapping_mode: Union[Unset, None, str] = UNSET
    tonemapping_range: Union[Unset, None, str] = UNSET
    tonemapping_desat: Union[Unset, float] = UNSET
    tonemapping_peak: Union[Unset, float] = UNSET
    tonemapping_param: Union[Unset, float] = UNSET
    vpp_tonemapping_brightness: Union[Unset, float] = UNSET
    vpp_tonemapping_contrast: Union[Unset, float] = UNSET
    h264_crf: Union[Unset, int] = UNSET
    h265_crf: Union[Unset, int] = UNSET
    encoder_preset: Union[Unset, None, str] = UNSET
    deinterlace_double_rate: Union[Unset, bool] = UNSET
    deinterlace_method: Union[Unset, None, str] = UNSET
    enable_decoding_color_depth_10_hevc: Union[Unset, bool] = UNSET
    enable_decoding_color_depth_10_vp_9: Union[Unset, bool] = UNSET
    enable_enhanced_nvdec_decoder: Union[Unset, bool] = UNSET
    prefer_system_native_hw_decoder: Union[Unset, bool] = UNSET
    enable_intel_low_power_h264_hw_encoder: Union[Unset, bool] = UNSET
    enable_intel_low_power_hevc_hw_encoder: Union[Unset, bool] = UNSET
    enable_hardware_encoding: Union[Unset, bool] = UNSET
    allow_hevc_encoding: Union[Unset, bool] = UNSET
    enable_subtitle_extraction: Union[Unset, bool] = UNSET
    hardware_decoding_codecs: Union[Unset, None, List[str]] = UNSET
    allow_on_demand_metadata_based_keyframe_extraction_for_extensions: Union[Unset, None, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        encoding_thread_count = self.encoding_thread_count
        transcoding_temp_path = self.transcoding_temp_path
        fallback_font_path = self.fallback_font_path
        enable_fallback_font = self.enable_fallback_font
        down_mix_audio_boost = self.down_mix_audio_boost
        max_muxing_queue_size = self.max_muxing_queue_size
        enable_throttling = self.enable_throttling
        throttle_delay_seconds = self.throttle_delay_seconds
        hardware_acceleration_type = self.hardware_acceleration_type
        encoder_app_path = self.encoder_app_path
        encoder_app_path_display = self.encoder_app_path_display
        vaapi_device = self.vaapi_device
        enable_tonemapping = self.enable_tonemapping
        enable_vpp_tonemapping = self.enable_vpp_tonemapping
        tonemapping_algorithm = self.tonemapping_algorithm
        tonemapping_mode = self.tonemapping_mode
        tonemapping_range = self.tonemapping_range
        tonemapping_desat = self.tonemapping_desat
        tonemapping_peak = self.tonemapping_peak
        tonemapping_param = self.tonemapping_param
        vpp_tonemapping_brightness = self.vpp_tonemapping_brightness
        vpp_tonemapping_contrast = self.vpp_tonemapping_contrast
        h264_crf = self.h264_crf
        h265_crf = self.h265_crf
        encoder_preset = self.encoder_preset
        deinterlace_double_rate = self.deinterlace_double_rate
        deinterlace_method = self.deinterlace_method
        enable_decoding_color_depth_10_hevc = self.enable_decoding_color_depth_10_hevc
        enable_decoding_color_depth_10_vp_9 = self.enable_decoding_color_depth_10_vp_9
        enable_enhanced_nvdec_decoder = self.enable_enhanced_nvdec_decoder
        prefer_system_native_hw_decoder = self.prefer_system_native_hw_decoder
        enable_intel_low_power_h264_hw_encoder = self.enable_intel_low_power_h264_hw_encoder
        enable_intel_low_power_hevc_hw_encoder = self.enable_intel_low_power_hevc_hw_encoder
        enable_hardware_encoding = self.enable_hardware_encoding
        allow_hevc_encoding = self.allow_hevc_encoding
        enable_subtitle_extraction = self.enable_subtitle_extraction
        hardware_decoding_codecs: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.hardware_decoding_codecs, Unset):
            if self.hardware_decoding_codecs is None:
                hardware_decoding_codecs = None
            else:
                hardware_decoding_codecs = self.hardware_decoding_codecs

        allow_on_demand_metadata_based_keyframe_extraction_for_extensions: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.allow_on_demand_metadata_based_keyframe_extraction_for_extensions, Unset):
            if self.allow_on_demand_metadata_based_keyframe_extraction_for_extensions is None:
                allow_on_demand_metadata_based_keyframe_extraction_for_extensions = None
            else:
                allow_on_demand_metadata_based_keyframe_extraction_for_extensions = (
                    self.allow_on_demand_metadata_based_keyframe_extraction_for_extensions
                )

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if encoding_thread_count is not UNSET:
            field_dict["EncodingThreadCount"] = encoding_thread_count
        if transcoding_temp_path is not UNSET:
            field_dict["TranscodingTempPath"] = transcoding_temp_path
        if fallback_font_path is not UNSET:
            field_dict["FallbackFontPath"] = fallback_font_path
        if enable_fallback_font is not UNSET:
            field_dict["EnableFallbackFont"] = enable_fallback_font
        if down_mix_audio_boost is not UNSET:
            field_dict["DownMixAudioBoost"] = down_mix_audio_boost
        if max_muxing_queue_size is not UNSET:
            field_dict["MaxMuxingQueueSize"] = max_muxing_queue_size
        if enable_throttling is not UNSET:
            field_dict["EnableThrottling"] = enable_throttling
        if throttle_delay_seconds is not UNSET:
            field_dict["ThrottleDelaySeconds"] = throttle_delay_seconds
        if hardware_acceleration_type is not UNSET:
            field_dict["HardwareAccelerationType"] = hardware_acceleration_type
        if encoder_app_path is not UNSET:
            field_dict["EncoderAppPath"] = encoder_app_path
        if encoder_app_path_display is not UNSET:
            field_dict["EncoderAppPathDisplay"] = encoder_app_path_display
        if vaapi_device is not UNSET:
            field_dict["VaapiDevice"] = vaapi_device
        if enable_tonemapping is not UNSET:
            field_dict["EnableTonemapping"] = enable_tonemapping
        if enable_vpp_tonemapping is not UNSET:
            field_dict["EnableVppTonemapping"] = enable_vpp_tonemapping
        if tonemapping_algorithm is not UNSET:
            field_dict["TonemappingAlgorithm"] = tonemapping_algorithm
        if tonemapping_mode is not UNSET:
            field_dict["TonemappingMode"] = tonemapping_mode
        if tonemapping_range is not UNSET:
            field_dict["TonemappingRange"] = tonemapping_range
        if tonemapping_desat is not UNSET:
            field_dict["TonemappingDesat"] = tonemapping_desat
        if tonemapping_peak is not UNSET:
            field_dict["TonemappingPeak"] = tonemapping_peak
        if tonemapping_param is not UNSET:
            field_dict["TonemappingParam"] = tonemapping_param
        if vpp_tonemapping_brightness is not UNSET:
            field_dict["VppTonemappingBrightness"] = vpp_tonemapping_brightness
        if vpp_tonemapping_contrast is not UNSET:
            field_dict["VppTonemappingContrast"] = vpp_tonemapping_contrast
        if h264_crf is not UNSET:
            field_dict["H264Crf"] = h264_crf
        if h265_crf is not UNSET:
            field_dict["H265Crf"] = h265_crf
        if encoder_preset is not UNSET:
            field_dict["EncoderPreset"] = encoder_preset
        if deinterlace_double_rate is not UNSET:
            field_dict["DeinterlaceDoubleRate"] = deinterlace_double_rate
        if deinterlace_method is not UNSET:
            field_dict["DeinterlaceMethod"] = deinterlace_method
        if enable_decoding_color_depth_10_hevc is not UNSET:
            field_dict["EnableDecodingColorDepth10Hevc"] = enable_decoding_color_depth_10_hevc
        if enable_decoding_color_depth_10_vp_9 is not UNSET:
            field_dict["EnableDecodingColorDepth10Vp9"] = enable_decoding_color_depth_10_vp_9
        if enable_enhanced_nvdec_decoder is not UNSET:
            field_dict["EnableEnhancedNvdecDecoder"] = enable_enhanced_nvdec_decoder
        if prefer_system_native_hw_decoder is not UNSET:
            field_dict["PreferSystemNativeHwDecoder"] = prefer_system_native_hw_decoder
        if enable_intel_low_power_h264_hw_encoder is not UNSET:
            field_dict["EnableIntelLowPowerH264HwEncoder"] = enable_intel_low_power_h264_hw_encoder
        if enable_intel_low_power_hevc_hw_encoder is not UNSET:
            field_dict["EnableIntelLowPowerHevcHwEncoder"] = enable_intel_low_power_hevc_hw_encoder
        if enable_hardware_encoding is not UNSET:
            field_dict["EnableHardwareEncoding"] = enable_hardware_encoding
        if allow_hevc_encoding is not UNSET:
            field_dict["AllowHevcEncoding"] = allow_hevc_encoding
        if enable_subtitle_extraction is not UNSET:
            field_dict["EnableSubtitleExtraction"] = enable_subtitle_extraction
        if hardware_decoding_codecs is not UNSET:
            field_dict["HardwareDecodingCodecs"] = hardware_decoding_codecs
        if allow_on_demand_metadata_based_keyframe_extraction_for_extensions is not UNSET:
            field_dict[
                "AllowOnDemandMetadataBasedKeyframeExtractionForExtensions"
            ] = allow_on_demand_metadata_based_keyframe_extraction_for_extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        encoding_thread_count = d.pop("EncodingThreadCount", UNSET)

        transcoding_temp_path = d.pop("TranscodingTempPath", UNSET)

        fallback_font_path = d.pop("FallbackFontPath", UNSET)

        enable_fallback_font = d.pop("EnableFallbackFont", UNSET)

        down_mix_audio_boost = d.pop("DownMixAudioBoost", UNSET)

        max_muxing_queue_size = d.pop("MaxMuxingQueueSize", UNSET)

        enable_throttling = d.pop("EnableThrottling", UNSET)

        throttle_delay_seconds = d.pop("ThrottleDelaySeconds", UNSET)

        hardware_acceleration_type = d.pop("HardwareAccelerationType", UNSET)

        encoder_app_path = d.pop("EncoderAppPath", UNSET)

        encoder_app_path_display = d.pop("EncoderAppPathDisplay", UNSET)

        vaapi_device = d.pop("VaapiDevice", UNSET)

        enable_tonemapping = d.pop("EnableTonemapping", UNSET)

        enable_vpp_tonemapping = d.pop("EnableVppTonemapping", UNSET)

        tonemapping_algorithm = d.pop("TonemappingAlgorithm", UNSET)

        tonemapping_mode = d.pop("TonemappingMode", UNSET)

        tonemapping_range = d.pop("TonemappingRange", UNSET)

        tonemapping_desat = d.pop("TonemappingDesat", UNSET)

        tonemapping_peak = d.pop("TonemappingPeak", UNSET)

        tonemapping_param = d.pop("TonemappingParam", UNSET)

        vpp_tonemapping_brightness = d.pop("VppTonemappingBrightness", UNSET)

        vpp_tonemapping_contrast = d.pop("VppTonemappingContrast", UNSET)

        h264_crf = d.pop("H264Crf", UNSET)

        h265_crf = d.pop("H265Crf", UNSET)

        encoder_preset = d.pop("EncoderPreset", UNSET)

        deinterlace_double_rate = d.pop("DeinterlaceDoubleRate", UNSET)

        deinterlace_method = d.pop("DeinterlaceMethod", UNSET)

        enable_decoding_color_depth_10_hevc = d.pop("EnableDecodingColorDepth10Hevc", UNSET)

        enable_decoding_color_depth_10_vp_9 = d.pop("EnableDecodingColorDepth10Vp9", UNSET)

        enable_enhanced_nvdec_decoder = d.pop("EnableEnhancedNvdecDecoder", UNSET)

        prefer_system_native_hw_decoder = d.pop("PreferSystemNativeHwDecoder", UNSET)

        enable_intel_low_power_h264_hw_encoder = d.pop("EnableIntelLowPowerH264HwEncoder", UNSET)

        enable_intel_low_power_hevc_hw_encoder = d.pop("EnableIntelLowPowerHevcHwEncoder", UNSET)

        enable_hardware_encoding = d.pop("EnableHardwareEncoding", UNSET)

        allow_hevc_encoding = d.pop("AllowHevcEncoding", UNSET)

        enable_subtitle_extraction = d.pop("EnableSubtitleExtraction", UNSET)

        hardware_decoding_codecs = cast(List[str], d.pop("HardwareDecodingCodecs", UNSET))

        allow_on_demand_metadata_based_keyframe_extraction_for_extensions = cast(
            List[str], d.pop("AllowOnDemandMetadataBasedKeyframeExtractionForExtensions", UNSET)
        )

        encoding_options = cls(
            encoding_thread_count=encoding_thread_count,
            transcoding_temp_path=transcoding_temp_path,
            fallback_font_path=fallback_font_path,
            enable_fallback_font=enable_fallback_font,
            down_mix_audio_boost=down_mix_audio_boost,
            max_muxing_queue_size=max_muxing_queue_size,
            enable_throttling=enable_throttling,
            throttle_delay_seconds=throttle_delay_seconds,
            hardware_acceleration_type=hardware_acceleration_type,
            encoder_app_path=encoder_app_path,
            encoder_app_path_display=encoder_app_path_display,
            vaapi_device=vaapi_device,
            enable_tonemapping=enable_tonemapping,
            enable_vpp_tonemapping=enable_vpp_tonemapping,
            tonemapping_algorithm=tonemapping_algorithm,
            tonemapping_mode=tonemapping_mode,
            tonemapping_range=tonemapping_range,
            tonemapping_desat=tonemapping_desat,
            tonemapping_peak=tonemapping_peak,
            tonemapping_param=tonemapping_param,
            vpp_tonemapping_brightness=vpp_tonemapping_brightness,
            vpp_tonemapping_contrast=vpp_tonemapping_contrast,
            h264_crf=h264_crf,
            h265_crf=h265_crf,
            encoder_preset=encoder_preset,
            deinterlace_double_rate=deinterlace_double_rate,
            deinterlace_method=deinterlace_method,
            enable_decoding_color_depth_10_hevc=enable_decoding_color_depth_10_hevc,
            enable_decoding_color_depth_10_vp_9=enable_decoding_color_depth_10_vp_9,
            enable_enhanced_nvdec_decoder=enable_enhanced_nvdec_decoder,
            prefer_system_native_hw_decoder=prefer_system_native_hw_decoder,
            enable_intel_low_power_h264_hw_encoder=enable_intel_low_power_h264_hw_encoder,
            enable_intel_low_power_hevc_hw_encoder=enable_intel_low_power_hevc_hw_encoder,
            enable_hardware_encoding=enable_hardware_encoding,
            allow_hevc_encoding=allow_hevc_encoding,
            enable_subtitle_extraction=enable_subtitle_extraction,
            hardware_decoding_codecs=hardware_decoding_codecs,
            allow_on_demand_metadata_based_keyframe_extraction_for_extensions=allow_on_demand_metadata_based_keyframe_extraction_for_extensions,
        )

        return encoding_options



from enum import Enum


class HardwareEncodingType(str, Enum):
    AMF = "AMF"
    NVENC = "NVENC"
    QSV = "QSV"
    V4L2M2M = "V4L2M2M"
    VAAPI = "VAAPI"
    VIDEOTOOLBOX = "VideoToolBox"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class FFmpegLocation(str, Enum):
    CUSTOM = "Custom"
    NOTFOUND = "NotFound"
    SETBYARGUMENT = "SetByArgument"
    SYSTEM = "System"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.general_command_type import GeneralCommandType
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    id: Union[Unset, None, str] = UNSET,
    playable_media_types: Union[Unset, None, List[str]] = UNSET,
    supported_commands: Union[Unset, None, List[GeneralCommandType]] = UNSET,
    supports_media_control: Union[Unset, None, bool] = False,
    supports_sync: Union[Unset, None, bool] = False,
    supports_persistent_identifier: Union[Unset, None, bool] = True,
) -> Dict[str, Any]:
    pass

    params: Dict[str, Any] = {}
    params["id"] = id

    json_playable_media_types: Union[Unset, None, List[str]] = UNSET
    if not isinstance(playable_media_types, Unset):
        if playable_media_types is None:
            json_playable_media_types = None
        else:
            json_playable_media_types = playable_media_types

    params["playableMediaTypes"] = json_playable_media_types

    json_supported_commands: Union[Unset, None, List[str]] = UNSET
    if not isinstance(supported_commands, Unset):
        if supported_commands is None:
            json_supported_commands = None
        else:
            json_supported_commands = []
            for supported_commands_item_data in supported_commands:
                supported_commands_item = supported_commands_item_data.value

                json_supported_commands.append(supported_commands_item)

    params["supportedCommands"] = json_supported_commands

    params["supportsMediaControl"] = supports_media_control

    params["supportsSync"] = supports_sync

    params["supportsPersistentIdentifier"] = supports_persistent_identifier

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "post",
        "url": "/Sessions/Capabilities",
        "params": params,
    }


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.NO_CONTENT:
        return None
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        return None
    if response.status_code == HTTPStatus.FORBIDDEN:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    id: Union[Unset, None, str] = UNSET,
    playable_media_types: Union[Unset, None, List[str]] = UNSET,
    supported_commands: Union[Unset, None, List[GeneralCommandType]] = UNSET,
    supports_media_control: Union[Unset, None, bool] = False,
    supports_sync: Union[Unset, None, bool] = False,
    supports_persistent_identifier: Union[Unset, None, bool] = True,
) -> Response[Any]:
    """Updates capabilities for a device.

    Args:
        id (Union[Unset, None, str]):
        playable_media_types (Union[Unset, None, List[str]]):
        supported_commands (Union[Unset, None, List[GeneralCommandType]]):
        supports_media_control (Union[Unset, None, bool]):
        supports_sync (Union[Unset, None, bool]):
        supports_persistent_identifier (Union[Unset, None, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        playable_media_types=playable_media_types,
        supported_commands=supported_commands,
        supports_media_control=supports_media_control,
        supports_sync=supports_sync,
        supports_persistent_identifier=supports_persistent_identifier,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    id: Union[Unset, None, str] = UNSET,
    playable_media_types: Union[Unset, None, List[str]] = UNSET,
    supported_commands: Union[Unset, None, List[GeneralCommandType]] = UNSET,
    supports_media_control: Union[Unset, None, bool] = False,
    supports_sync: Union[Unset, None, bool] = False,
    supports_persistent_identifier: Union[Unset, None, bool] = True,
) -> Response[Any]:
    """Updates capabilities for a device.

    Args:
        id (Union[Unset, None, str]):
        playable_media_types (Union[Unset, None, List[str]]):
        supported_commands (Union[Unset, None, List[GeneralCommandType]]):
        supports_media_control (Union[Unset, None, bool]):
        supports_sync (Union[Unset, None, bool]):
        supports_persistent_identifier (Union[Unset, None, bool]):  Default: True.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        playable_media_types=playable_media_types,
        supported_commands=supported_commands,
        supports_media_control=supports_media_control,
        supports_sync=supports_sync,
        supports_persistent_identifier=supports_persistent_identifier,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from enum import Enum


class LogLevel(str, Enum):
    CRITICAL = "Critical"
    DEBUG = "Debug"
    ERROR = "Error"
    INFORMATION = "Information"
    NONE = "None"
    TRACE = "Trace"
    WARNING = "Warning"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ImageType(str, Enum):
    ART = "Art"
    BACKDROP = "Backdrop"
    BANNER = "Banner"
    BOX = "Box"
    BOXREAR = "BoxRear"
    CHAPTER = "Chapter"
    DISC = "Disc"
    LOGO = "Logo"
    MENU = "Menu"
    PRIMARY = "Primary"
    PROFILE = "Profile"
    SCREENSHOT = "Screenshot"
    THUMB = "Thumb"

    def __str__(self) -> str:
        return str(self.value)
